@@ -47,6 +47,9 @@ struct _CalClientPrivate {
 	 */
 	char *uri;
 
+	/* Email address associated with this calendar, or NULL */
+	char *email_address;
+
 	/* The calendar factories we are contacting */
 	GList *factories;
 
@@ -83,6 +86,7 @@ enum {
 	CAL_SET_MODE,
 	OBJ_UPDATED,
 	OBJ_REMOVED,
+	BACKEND_ERROR,
 	CATEGORIES_CHANGED,
 	FORGET_PASSWORD,
 	BACKEND_DIED,
@@ -185,6 +189,14 @@ cal_client_class_init (CalClientClass *class)
 				gtk_marshal_NONE__STRING,
 				GTK_TYPE_NONE, 1,
 				GTK_TYPE_STRING);
+	cal_client_signals[BACKEND_ERROR] =
+		gtk_signal_new ("backend_error",
+				GTK_RUN_FIRST,
+				object_class->type,
+				GTK_SIGNAL_OFFSET (CalClientClass, backend_error),
+				gtk_marshal_NONE__STRING,
+				GTK_TYPE_NONE, 1,
+				GTK_TYPE_STRING);
 	cal_client_signals[CATEGORIES_CHANGED] =
 		gtk_signal_new ("categories_changed",
 				GTK_RUN_FIRST,
@@ -232,6 +244,7 @@ cal_client_init (CalClient *client)
 
 	priv->load_state = CAL_CLIENT_LOAD_NOT_LOADED;
 	priv->uri = NULL;
+	priv->email_address = NULL;
 	priv->factories = NULL;
 	priv->timezones = g_hash_table_new (g_str_hash, g_str_equal);
 	priv->w_client = NULL;
@@ -363,6 +376,11 @@ cal_client_destroy (GtkObject *object)
 		priv->uri = NULL;
 	}
 
+	if (priv->email_address) {
+		g_free (priv->email_address);
+		priv->email_address = NULL;
+	}
+
 	g_hash_table_foreach (priv->timezones, free_timezone, NULL);
 	g_hash_table_destroy (priv->timezones);
 	priv->timezones = NULL;
@@ -552,6 +570,16 @@ obj_removed_cb (CalListener *listener, const GNOME_Evolution_Calendar_CalObjUID
 	gtk_signal_emit (GTK_OBJECT (client), cal_client_signals[OBJ_REMOVED], uid);
 }
 
+/* Handle the error_occurred signal from the listener */
+static void
+backend_error_cb (CalListener *listener, const char *message, gpointer data)
+{
+	CalClient *client;
+
+	client = CAL_CLIENT (data);
+	gtk_signal_emit (GTK_OBJECT (client), cal_client_signals[BACKEND_ERROR], message);
+}
+
 /* Handle the categories_changed signal from the listener */
 static void
 categories_changed_cb (CalListener *listener, const GNOME_Evolution_Calendar_StringSeq *categories,
@@ -743,6 +771,7 @@ real_open_calendar (CalClient *client, const char *str_uri, gboolean only_if_exi
 					   cal_set_mode_cb,
 					   obj_updated_cb,
 					   obj_removed_cb,
+					   backend_error_cb,
 					   categories_changed_cb,
 					   client);
 	if (!priv->listener) {
@@ -1009,6 +1038,76 @@ cal_client_get_uri (CalClient *client)
 	return priv->uri;
 }
 
+/**
+ * cal_client_is_read_only:
+ * @client: A calendar client.
+ *
+ * Queries whether the calendar client can perform modifications
+ * on the calendar or not.
+ *
+ * Return value: TRUE if the calendar is read-only, FALSE otherwise.
+ */
+gboolean
+cal_client_is_read_only (CalClient *client)
+{
+	CalClientPrivate *priv;
+	CORBA_Environment ev;
+	CORBA_boolean read_only;
+
+	g_return_val_if_fail (client != NULL, NULL);
+	g_return_val_if_fail (IS_CAL_CLIENT (client), NULL);
+
+	priv = client->priv;
+
+	g_return_val_if_fail (priv->load_state == CAL_CLIENT_LOAD_LOADED, FALSE);
+
+	CORBA_exception_init (&ev);
+	read_only = GNOME_Evolution_Calendar_Cal_isReadOnly (priv->cal, &ev);
+	if (BONOBO_EX (&ev)) {
+		g_message ("cal_client_is_read_only: could not call isReadOnly method");
+	}
+	CORBA_exception_free (&ev);
+
+	return read_only;
+}
+
+/**
+ * cal_client_get_email_address:
+ * @client: A calendar client.
+ * 
+ * Queries the email address associated with a calendar client.
+ * 
+ * Return value: The email address associated with the calendar that
+ * is loaded or being loaded, or %NULL if the client has not started a
+ * load request yet or the calendar has no associated email address.
+ **/
+const char *
+cal_client_get_email_address (CalClient *client)
+{
+	CalClientPrivate *priv;
+
+	g_return_val_if_fail (client != NULL, NULL);
+	g_return_val_if_fail (IS_CAL_CLIENT (client), NULL);
+
+	priv = client->priv;
+	g_return_val_if_fail (priv->load_state == CAL_CLIENT_LOAD_LOADED, NULL);
+
+	if (priv->email_address == NULL) {
+		CORBA_Environment ev;
+		CORBA_char *email_address;
+
+		CORBA_exception_init (&ev);
+		email_address = GNOME_Evolution_Calendar_Cal_getEmailAddress (priv->cal, &ev);
+		if (!BONOBO_EX (&ev)) {
+			priv->email_address = g_strdup (email_address);
+			CORBA_free (email_address);
+		}
+		CORBA_exception_free (&ev);
+	}
+
+	return priv->email_address;
+}
+
 /* Converts our representation of a calendar component type into its CORBA representation */
 static GNOME_Evolution_Calendar_CalObjType
 corba_obj_type (CalObjType type)
