@@ -65,6 +65,7 @@ void conduit_destroy_gpilot_conduit (GnomePilotConduit*);
 
 typedef struct _ECalLocalRecord ECalLocalRecord;
 typedef struct _ECalConduitCfg ECalConduitCfg;
+typedef struct _ECalConduitGui ECalConduitGui;
 typedef struct _ECalConduitContext ECalConduitContext;
 
 /* Local Record */
@@ -97,6 +98,8 @@ struct _ECalConduitCfg {
 	GnomePilotConduitSyncType  sync_type;
 
 	gboolean secret;
+	gboolean multi_day_split;
+	
 	gchar *last_uri;
 };
 
@@ -127,9 +130,10 @@ calconduit_load_configuration (guint32 pilot_id)
 	gnome_config_push_prefix (prefix);
 
 	c->secret = gnome_config_get_bool ("secret=FALSE");
+	c->multi_day_split = gnome_config_get_bool ("multi_day_split=TRUE");
 	c->last_uri = gnome_config_get_string ("last_uri");
 
-	gnome_config_pop_prefix ();
+	gnome_config_pop_prefix (); 
 
 	return c;
 }
@@ -143,6 +147,7 @@ calconduit_save_configuration (ECalConduitCfg *c)
 	gnome_config_push_prefix (prefix);
 
 	gnome_config_set_bool ("secret", c->secret);
+	gnome_config_set_bool ("multi_day_split", c->multi_day_split);
 	gnome_config_set_string ("last_uri", c->last_uri);
 
 	gnome_config_pop_prefix ();
@@ -162,6 +167,7 @@ calconduit_dupe_configuration (ECalConduitCfg *c)
 	retval->pilot_id = c->pilot_id;
 	retval->sync_type = c->sync_type;
 	retval->secret = c->secret;
+	retval->multi_day_split = c->multi_day_split;
 	retval->last_uri = g_strdup (c->last_uri);
 	
 	return retval;
@@ -176,12 +182,68 @@ calconduit_destroy_configuration (ECalConduitCfg *c)
 	g_free (c);
 }
 
+/* Gui */
+struct _ECalConduitGui {
+	GtkWidget *multi_day_split;
+};
+
+static ECalConduitGui *
+e_cal_gui_new (EPilotSettings *ps) 
+{
+	ECalConduitGui *gui;
+	GtkWidget *lbl;
+	gint rows;
+	
+	g_return_val_if_fail (ps != NULL, NULL);
+	g_return_val_if_fail (E_IS_PILOT_SETTINGS (ps), NULL);
+
+	gtk_table_resize (GTK_TABLE (ps), E_PILOT_SETTINGS_TABLE_ROWS + 1, E_PILOT_SETTINGS_TABLE_COLS);
+
+	gui = g_new0 (ECalConduitGui, 1);
+
+	rows = E_PILOT_SETTINGS_TABLE_ROWS;
+	lbl = gtk_label_new (_("Split Multi-Day Events:"));
+	gui->multi_day_split = gtk_check_button_new ();
+	gtk_table_attach_defaults (GTK_TABLE (ps), lbl, 0, 1, rows, rows + 1);
+	gtk_table_attach_defaults (GTK_TABLE (ps), gui->multi_day_split, 1, 2, rows, rows + 1);
+	gtk_widget_show (lbl);
+	gtk_widget_show (gui->multi_day_split);
+	
+	return gui;
+}
+
+static void
+e_cal_gui_fill_widgets (ECalConduitGui *gui, ECalConduitCfg *cfg) 
+{
+	g_return_if_fail (gui != NULL);
+	g_return_if_fail (cfg != NULL);
+
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (gui->multi_day_split),
+				      cfg->multi_day_split);
+}
+
+static void
+e_cal_gui_fill_config (ECalConduitGui *gui, ECalConduitCfg *cfg) 
+{
+	g_return_if_fail (gui != NULL);
+	g_return_if_fail (cfg != NULL);
+
+	cfg->multi_day_split = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (gui->multi_day_split));
+}
+
+static void
+e_cal_gui_destroy (ECalConduitGui *gui) 
+{
+	g_free (gui);
+}
+
 /* Context */
 struct _ECalConduitContext {
-	ECalConduitCfg *cfg;
 	GnomePilotDBInfo *dbi;
 
+	ECalConduitCfg *cfg;
 	ECalConduitCfg *new_cfg;
+	ECalConduitGui *gui;
 	GtkWidget *ps;
 	
 	struct AppointmentAppInfo ai;
@@ -237,6 +299,10 @@ e_calendar_context_destroy (ECalConduitContext *ctxt)
 
 	if (ctxt->cfg != NULL)
 		calconduit_destroy_configuration (ctxt->cfg);
+	if (ctxt->new_cfg != NULL)
+		calconduit_destroy_configuration (ctxt->new_cfg);
+	if (ctxt->gui != NULL)
+		e_cal_gui_destroy (ctxt->gui);
 
 	if (ctxt->client != NULL)
 		gtk_object_unref (GTK_OBJECT (ctxt->client));
@@ -497,6 +563,7 @@ process_multi_day (ECalConduitContext *ctxt, CalClientChange *ccc, GList **multi
 	time_t event_start, event_end, day_end;
 	struct icaltimetype *old_start_value, *old_end_value;
 	const char *uid;
+	gboolean is_date = FALSE;
 	gboolean last = FALSE;
 	gboolean ret = TRUE;
 
@@ -525,11 +592,14 @@ process_multi_day (ECalConduitContext *ctxt, CalClientChange *ccc, GList **multi
 	if (day_end >= event_end) {
 		ret = FALSE;
 		goto cleanup;
-	} else if (cal_component_has_recurrences (ccc->comp)) {
+	} else if (cal_component_has_recurrences (ccc->comp) || !ctxt->cfg->multi_day_split) {
 		ret = TRUE;
 		goto cleanup;
 	}
 
+	if (dt_start.value->is_date && dt_end.value->is_date)
+		is_date = TRUE;
+	
 	old_start_value = dt_start.value;
  	old_end_value = dt_end.value;
 	while (!last) {
@@ -545,11 +615,11 @@ process_multi_day (ECalConduitContext *ctxt, CalClientChange *ccc, GList **multi
 
 		cal_component_set_uid (clone, new_uid);
 
-		start_value = icaltime_from_timet_with_zone (event_start, FALSE, tz_start);
+		start_value = icaltime_from_timet_with_zone (event_start, is_date, tz_start);
 		dt_start.value = &start_value;
 		cal_component_set_dtstart (clone, &dt_start);
 		
-		end_value = icaltime_from_timet_with_zone (day_end, FALSE, tz_end);
+		end_value = icaltime_from_timet_with_zone (day_end, is_date, tz_end);
 		dt_end.value = &end_value;
 		cal_component_set_dtend (clone, &dt_end);
 
@@ -934,7 +1004,7 @@ comp_from_remote_record (GnomePilotConduitSyncAbs *conduit,
 	struct icaltimetype now = icaltime_from_timet_with_zone (time (NULL), FALSE, timezone), it;
 	struct icalrecurrencetype recur;
 	CalComponentText summary = {NULL, NULL};
-	CalComponentDateTime dt = {NULL, icaltimezone_get_tzid (timezone)};
+	CalComponentDateTime dt = {NULL, NULL};
 	GSList *edl = NULL;	
 	char *txt;
 	int pos, i;
@@ -974,21 +1044,26 @@ comp_from_remote_record (GnomePilotConduitSyncAbs *conduit,
 		free (txt);
 	} 
 
-	if (!is_empty_time (appt.begin)) {
-		it = tm_to_icaltimetype (&appt.begin, FALSE);
+	if (appt.event && !is_empty_time (appt.begin)) {
+		it = tm_to_icaltimetype (&appt.begin, TRUE);
 		dt.value = &it;
+		dt.tzid = NULL;
 		cal_component_set_dtstart (comp, &dt);
-	}
+		cal_component_set_dtend (comp, &dt);
+	} else {
+		dt.tzid = icaltimezone_get_tzid (timezone);
 
-	if (appt.event) {
-		it = tm_to_icaltimetype (&appt.begin, FALSE);
-		icaltime_adjust (&it, 1, 0, 0, 0);
-		dt.value = &it;
-		cal_component_set_dtend (comp, &dt);
-	} else if (!is_empty_time (appt.end)) {
-		it = tm_to_icaltimetype (&appt.end, FALSE);
-		dt.value = &it;
-		cal_component_set_dtend (comp, &dt);
+		if (!is_empty_time (appt.begin)) {
+			it = tm_to_icaltimetype (&appt.begin, FALSE);
+			dt.value = &it;
+			cal_component_set_dtstart (comp, &dt);
+		}
+
+		if (!is_empty_time (appt.end)) {		
+			it = tm_to_icaltimetype (&appt.end, FALSE);
+			dt.value = &it;
+			cal_component_set_dtend (comp, &dt);
+		}
 	}
 
 	/* Recurrence information */
@@ -1650,6 +1725,8 @@ fill_widgets (ECalConduitContext *ctxt)
 {
 	e_pilot_settings_set_secret (E_PILOT_SETTINGS (ctxt->ps),
 				     ctxt->cfg->secret);
+
+	e_cal_gui_fill_widgets (ctxt->gui, ctxt->cfg);
 }
 
 static gint
@@ -1660,6 +1737,8 @@ create_settings_window (GnomePilotConduit *conduit,
 	LOG ("create_settings_window");
 
 	ctxt->ps = e_pilot_settings_new ();
+	ctxt->gui = e_cal_gui_new (E_PILOT_SETTINGS (ctxt->ps));
+
 	gtk_container_add (GTK_CONTAINER (parent), ctxt->ps);
 	gtk_widget_show (ctxt->ps);
 
@@ -1682,7 +1761,8 @@ save_settings    (GnomePilotConduit *conduit, ECalConduitContext *ctxt)
 
 	ctxt->new_cfg->secret =
 		e_pilot_settings_get_secret (E_PILOT_SETTINGS (ctxt->ps));
-	
+	e_cal_gui_fill_config (ctxt->gui, ctxt->new_cfg);
+
 	calconduit_save_configuration (ctxt->new_cfg);
 }
 
