@@ -1554,18 +1554,11 @@ client_cal_opened_cb (CalClient *client, CalClientOpenStatus status, gpointer da
 {
 	GnomeCalendar *gcal;
 	GnomeCalendarPrivate *priv;
+	char *msg;
 
 	gcal = GNOME_CALENDAR (data);
 	priv = gcal->priv;
 
-	if (client == priv->client) {
-		e_week_view_set_status_message (E_WEEK_VIEW (priv->week_view), NULL);
-	}
-	else if (client == priv->task_pad_client) {
-		calendar_model_set_status_message (
-			e_calendar_table_get_model (E_CALENDAR_TABLE (priv->todo)), NULL);
-	}
-
 	switch (status) {
 	case CAL_CLIENT_OPEN_SUCCESS:
 		/* If this is the main CalClient, update the Date Navigator. */
@@ -1579,6 +1572,16 @@ client_cal_opened_cb (CalClient *client, CalClientOpenStatus status, gpointer da
 		}
 
 		/* add the alarms for this client */
+		msg = g_strdup_printf (_("Adding alarms for %s"), cal_client_get_uri (client));
+		if (client == priv->client) {
+			e_week_view_set_status_message (E_WEEK_VIEW (priv->week_view), msg);
+		}
+		else if (client == priv->task_pad_client) {
+			calendar_model_set_status_message (
+				e_calendar_table_get_model (E_CALENDAR_TABLE (priv->todo)), msg);
+		}
+		g_free (msg);
+
 		add_alarms (cal_client_get_uri (client));
 		break;
 
@@ -1603,6 +1606,14 @@ client_cal_opened_cb (CalClient *client, CalClientOpenStatus status, gpointer da
 		g_assert_not_reached ();
 		return;
 	}
+
+	if (client == priv->client) {
+		e_week_view_set_status_message (E_WEEK_VIEW (priv->week_view), NULL);
+	}
+	else if (client == priv->task_pad_client) {
+		calendar_model_set_status_message (
+			e_calendar_table_get_model (E_CALENDAR_TABLE (priv->todo)), NULL);
+	}
 }
 
 /* Duplicates an array of categories */
@@ -1719,6 +1730,22 @@ client_categories_changed_cb (CalClient *client, GPtrArray *categories, gpointer
 	free_categories (merged);
 }
 
+/* Callback when we get an error message from the backend */
+static void
+backend_error_cb (CalClient *client, const char *message, gpointer data)
+{
+	GnomeCalendar *gcal;
+	GnomeCalendarPrivate *priv;
+	char *errmsg;
+
+	gcal = GNOME_CALENDAR (data);
+	priv = gcal->priv;
+
+	errmsg = g_strdup_printf (_("Error on %s:\n %s"), cal_client_get_uri (client), message);
+	gnome_error_dialog_parented (errmsg, GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (gcal))));
+	g_free (errmsg);
+}
+
 /* Callback when the backend dies */
 static void
 backend_died_cb (CalClient *client, gpointer data)
@@ -1768,6 +1795,8 @@ gnome_calendar_construct (GnomeCalendar *gcal)
 
 	gtk_signal_connect (GTK_OBJECT (priv->client), "cal_opened",
 			    GTK_SIGNAL_FUNC (client_cal_opened_cb), gcal);
+	gtk_signal_connect (GTK_OBJECT (priv->client), "backend_error",
+			    GTK_SIGNAL_FUNC (backend_error_cb), gcal);
 	gtk_signal_connect (GTK_OBJECT (priv->client), "categories_changed",
 			    GTK_SIGNAL_FUNC (client_categories_changed_cb), gcal);
 	gtk_signal_connect (GTK_OBJECT (priv->client), "backend_died",
@@ -1790,7 +1819,9 @@ gnome_calendar_construct (GnomeCalendar *gcal)
 		return NULL;
 
 	gtk_signal_connect (GTK_OBJECT (priv->task_pad_client), "cal_opened",
-			    GTK_SIGNAL_FUNC (client_cal_opened_cb), gcal);
+			    GTK_SIGNAL_FUNC (client_cal_opened_cb), gcal);	
+	gtk_signal_connect (GTK_OBJECT (priv->client), "backend_error",
+			    GTK_SIGNAL_FUNC (backend_error_cb), gcal);
 	gtk_signal_connect (GTK_OBJECT (priv->task_pad_client), "categories_changed",
 			    GTK_SIGNAL_FUNC (client_categories_changed_cb), gcal);
 	gtk_signal_connect (GTK_OBJECT (priv->task_pad_client), "backend_died",
@@ -2197,14 +2228,13 @@ gnome_calendar_edit_object (GnomeCalendar *gcal, CalComponent *comp,
 	if (!ce) {
 		EventEditor *ee;
 
-		ee = event_editor_new ();
+		ee = event_editor_new (priv->client);
 		if (!ee) {
 			g_message ("gnome_calendar_edit_object(): Could not create the event editor");
 			return;
 		}
 		ce = COMP_EDITOR (ee);
 		
-		comp_editor_set_cal_client (ce, priv->client);
 		comp_editor_edit_comp (ce, comp);
 		if (meeting)
 			event_editor_show_meeting (ee);
@@ -2330,8 +2360,7 @@ gnome_calendar_new_task		(GnomeCalendar *gcal)
 
 	priv = gcal->priv;
 
-	tedit = task_editor_new ();
-	comp_editor_set_cal_client (COMP_EDITOR (tedit), priv->task_pad_client);
+	tedit = task_editor_new (priv->task_pad_client);
 
 	comp = cal_component_new ();
 	cal_component_set_new_vtype (comp, CAL_COMPONENT_TODO);
