@@ -5,30 +5,36 @@
  *  Authors: Michael Zucchi <notzed@ximian.com>
  *           Jeffrey Stedfast <fejj@ximian.com>
  *
- *  This program is free software; you can redistribute it and/or 
- *  modify it under the terms of the GNU General Public License as 
- *  published by the Free Software Foundation; either version 2 of the
- *  License, or (at your option) any later version.
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of version 2 of the GNU General Public
+ * License as published by the Free Software Foundation.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- *  USA
+ * You should have received a copy of the GNU General Public
+ * License along with this program; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+#include <errno.h>
+
 #include <string.h>
 #include <time.h>
 
 #include <glib.h>
+#include <gtk/gtk.h>
 
 #include "camel-filter-driver.h"
 #include "camel-filter-search.h"
@@ -67,10 +73,21 @@ struct _filter_rule {
 struct _CamelFilterDriverPrivate {
 	GHashTable *globals;       /* global variables */
 
-	CamelFolder *defaultfolder;	/* defualt folder */
+	CamelSession *session;
+
+	CamelFolder *defaultfolder;        /* defualt folder */
 	
-	CamelFilterStatusFunc *statusfunc; 	/* status callback */
-	void *statusdata;		/* status callback data */
+	CamelFilterStatusFunc *statusfunc; /* status callback */
+	void *statusdata;                  /* status callback data */
+	
+	CamelFilterShellFunc *shellfunc;    /* execute shell command callback */
+	void *shelldata;                    /* execute shell command callback data */
+	
+	CamelFilterPlaySoundFunc *playfunc; /* play-sound command callback */
+	void *playdata;                     /* play-sound command callback data */
+	
+	CamelFilterSystemBeepFunc *beep;    /* system beep callback */
+	void *beepdata;                     /* system beep callback data */
 	
 	/* for callback */
 	CamelFilterGetFolderFunc get_folder;
@@ -80,6 +97,7 @@ struct _CamelFilterDriverPrivate {
 	GHashTable *folders;       /* folders that message has been copied to */
 	int closed;		   /* close count */
 	GHashTable *forwards;      /* addresses that have been forwarded the message */
+	GHashTable *only_once;     /* actions to run only-once */
 	
 	gboolean terminated;       /* message processing was terminated */
 	gboolean deleted;          /* message was marked for deletion */
@@ -93,7 +111,7 @@ struct _CamelFilterDriverPrivate {
 	FILE *logfile;             /* log file */
 	
 	EDList rules;		   /* list of _filter_rule structs */
-
+	
 	CamelException *ex;
 	
 	/* evaluator */
@@ -118,7 +136,12 @@ static ESExpResult *do_move (struct _ESExp *f, int argc, struct _ESExpResult **a
 static ESExpResult *do_stop (struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFilterDriver *);
 static ESExpResult *do_colour (struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFilterDriver *);
 static ESExpResult *do_score (struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFilterDriver *);
-static ESExpResult *do_flag (struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFilterDriver *);
+static ESExpResult *set_flag (struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFilterDriver *);
+static ESExpResult *unset_flag (struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFilterDriver *);
+static ESExpResult *do_shell (struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFilterDriver *);
+static ESExpResult *do_beep (struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFilterDriver *);
+static ESExpResult *play_sound (struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFilterDriver *);
+static ESExpResult *do_only_once (struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFilterDriver *);
 
 /* these are our filter actions - each must have a callback */
 static struct {
@@ -127,31 +150,37 @@ static struct {
 	int type;		/* set to 1 if a function can perform shortcut evaluation, or
 				   doesn't execute everything, 0 otherwise */
 } symbols[] = {
-	{ "delete",          (ESExpFunc *) do_delete,    0 },
-	{ "forward-to",      (ESExpFunc *) mark_forward, 0 },
-	{ "copy-to",         (ESExpFunc *) do_copy,      0 },
-	{ "move-to",         (ESExpFunc *) do_move,      0 },
-	{ "stop",            (ESExpFunc *) do_stop,      0 },
-	{ "set-colour",      (ESExpFunc *) do_colour,    0 },
-	{ "set-score",       (ESExpFunc *) do_score,     0 },
-	{ "set-system-flag", (ESExpFunc *) do_flag,      0 }
+	{ "delete",            (ESExpFunc *) do_delete,    0 },
+	{ "forward-to",        (ESExpFunc *) mark_forward, 0 },
+	{ "copy-to",           (ESExpFunc *) do_copy,      0 },
+	{ "move-to",           (ESExpFunc *) do_move,      0 },
+	{ "stop",              (ESExpFunc *) do_stop,      0 },
+	{ "set-colour",        (ESExpFunc *) do_colour,    0 },
+	{ "set-score",         (ESExpFunc *) do_score,     0 },
+	{ "set-system-flag",   (ESExpFunc *) set_flag,     0 },
+	{ "unset-system-flag", (ESExpFunc *) unset_flag,   0 },
+	{ "shell",             (ESExpFunc *) do_shell,     0 },
+	{ "beep",              (ESExpFunc *) do_beep,      0 },
+	{ "play-sound",        (ESExpFunc *) play_sound,   0 },
+	{ "only-once",         (ESExpFunc *) do_only_once, 0 }
 };
 
 static CamelObjectClass *camel_filter_driver_parent;
 
-guint
+CamelType
 camel_filter_driver_get_type (void)
 {
 	static CamelType type = CAMEL_INVALID_TYPE;
 
 	if (type == CAMEL_INVALID_TYPE)	{
-		type = camel_type_register(CAMEL_OBJECT_TYPE, "CamelFilterDriver",
-					   sizeof(CamelFilterDriver),
-					   sizeof(CamelFilterDriverClass),
-					   (CamelObjectClassInitFunc)camel_filter_driver_class_init,
-					   NULL,
-					   (CamelObjectInitFunc)camel_filter_driver_init,
-					   (CamelObjectFinalizeFunc)camel_filter_driver_finalise);
+		type = camel_type_register (CAMEL_OBJECT_TYPE,
+					    "CamelFilterDriver",
+					    sizeof (CamelFilterDriver),
+					    sizeof (CamelFilterDriverClass),
+					    (CamelObjectClassInitFunc) camel_filter_driver_class_init,
+					    NULL,
+					    (CamelObjectInitFunc) camel_filter_driver_init,
+					    (CamelObjectFinalizeFunc) camel_filter_driver_finalise);
 	}
 	
 	return type;
@@ -188,6 +217,8 @@ camel_filter_driver_init (CamelFilterDriver *obj)
 	p->globals = g_hash_table_new (g_str_hash, g_str_equal);
 	
 	p->folders = g_hash_table_new (g_str_hash, g_str_equal);
+	
+	p->only_once = g_hash_table_new (g_str_hash, g_str_equal);
 }
 
 static void
@@ -210,7 +241,10 @@ camel_filter_driver_finalise (CamelObject *obj)
 	
 	g_hash_table_foreach (p->globals, free_hash_strings, driver);
 	g_hash_table_destroy (p->globals);
-
+	
+	g_hash_table_foreach (p->only_once, free_hash_strings, driver);
+	g_hash_table_destroy (p->only_once);
+	
 	e_sexp_unref(p->eval);
 	
 	if (p->defaultfolder) {
@@ -224,7 +258,9 @@ camel_filter_driver_finalise (CamelObject *obj)
 		g_free(node->name);
 		g_free(node);
 	}
-	
+
+	camel_object_unref(p->session);
+
 	g_free (p);
 }
 
@@ -234,9 +270,14 @@ camel_filter_driver_finalise (CamelObject *obj)
  * Return value: A new CamelFilterDriver object
  **/
 CamelFilterDriver *
-camel_filter_driver_new (void)
+camel_filter_driver_new (CamelSession *session)
 {
-	return CAMEL_FILTER_DRIVER (camel_object_new(camel_filter_driver_get_type ()));
+	CamelFilterDriver *d = (CamelFilterDriver *)camel_object_new(camel_filter_driver_get_type());
+
+	d->priv->session = session;
+	camel_object_ref((CamelObject *)session);
+
+	return d;
 }
 
 void
@@ -265,6 +306,33 @@ camel_filter_driver_set_status_func (CamelFilterDriver *d, CamelFilterStatusFunc
 	p->statusdata = data;
 }
 
+void
+camel_filter_driver_set_shell_func (CamelFilterDriver *d, CamelFilterShellFunc *func, void *data)
+{
+	struct _CamelFilterDriverPrivate *p = _PRIVATE (d);
+	
+	p->shellfunc = func;
+	p->shelldata = data;
+}
+
+void
+camel_filter_driver_set_play_sound_func (CamelFilterDriver *d, CamelFilterPlaySoundFunc *func, void *data)
+{
+	struct _CamelFilterDriverPrivate *p = _PRIVATE (d);
+	
+	p->playfunc = func;
+	p->playdata = data;
+}
+
+void
+camel_filter_driver_set_system_beep_func (CamelFilterDriver *d, CamelFilterSystemBeepFunc *func, void *data)
+{
+	struct _CamelFilterDriverPrivate *p = _PRIVATE (d);
+	
+	p->beep = func;
+	p->beepdata = data;
+}
+
 void
 camel_filter_driver_set_default_folder (CamelFilterDriver *d, CamelFolder *def)
 {
@@ -296,6 +364,30 @@ camel_filter_driver_add_rule(CamelFilterDriver *d, const char *name, const char
 	e_dlist_addtail(&p->rules, (EDListNode *)node);
 }
 
+int
+camel_filter_driver_remove_rule_by_name (CamelFilterDriver *d, const char *name)
+{
+	struct _CamelFilterDriverPrivate *p = _PRIVATE (d);
+	struct _filter_rule *node;
+	
+	node = (struct _filter_rule *) p->rules.head;
+	while (node->next) {
+		if (!strcmp (node->name, name)) {
+			e_dlist_remove ((EDListNode *) node);
+			g_free (node->match);
+			g_free (node->action);
+			g_free (node->name);
+			g_free (node);
+			
+			return 0;
+		}
+		
+		node = node->next;
+	}
+	
+	return -1;
+}
+
 static void
 report_status (CamelFilterDriver *driver, enum camel_filter_status_t status, int pc, const char *desc, ...)
 {
@@ -371,18 +463,30 @@ do_copy (struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFilterDriv
 			if (!outbox)
 				break;
 			
+			if (outbox == p->source)
+				break;
+			
 			if (p->uid && p->source && camel_folder_has_summary_capability (p->source)) {
 				GPtrArray *uids;
 				
 				uids = g_ptr_array_new ();
 				g_ptr_array_add (uids, (char *) p->uid);
-				camel_folder_copy_messages_to (p->source, uids, outbox, p->ex);
+				camel_folder_transfer_messages_to (p->source, uids, outbox, NULL, FALSE, p->ex);
 				g_ptr_array_free (uids, TRUE);
-			} else
-				camel_folder_append_message (outbox, p->message, p->info, p->ex);
+			} else {
+				if (p->message == NULL)
+					p->message = camel_folder_get_message (p->source, p->uid, p->ex);
+				
+				if (!p->message) {
+					/* FIXME: exception? */
+					continue;
+				}
+				
+				camel_folder_append_message (outbox, p->message, p->info, NULL, p->ex);
+			}
 			
 			if (!camel_exception_is_set (p->ex))
-				p->copied = TRUE;
+					p->copied = TRUE;
 			
 			camel_filter_driver_log (driver, FILTER_LOG_ACTION, "Copy to folder %s",
 						 folder);
@@ -418,10 +522,19 @@ do_move (struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFilterDriv
 				
 				uids = g_ptr_array_new ();
 				g_ptr_array_add (uids, (char *) p->uid);
-				camel_folder_copy_messages_to (p->source, uids, outbox, p->ex);
+				camel_folder_transfer_messages_to (p->source, uids, outbox, NULL, FALSE, p->ex);
 				g_ptr_array_free (uids, TRUE);
-			} else
-				camel_folder_append_message (outbox, p->message, p->info, p->ex);
+			} else {
+				if (p->message == NULL)
+					p->message = camel_folder_get_message (p->source, p->uid, p->ex);
+				
+				if (!p->message) {
+					/* FIXME: exception? */
+					continue;
+				}
+				
+				camel_folder_append_message (outbox, p->message, p->info, NULL, p->ex);
+			}
 			
 			if (!camel_exception_is_set (p->ex)) {
 				/* a 'move' is a copy & delete */
@@ -456,7 +569,7 @@ do_colour (struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFilterDr
 	
 	d(fprintf (stderr, "setting colour tag\n"));
 	if (argc > 0 && argv[0]->type == ESEXP_RES_STRING) {
-		if (p->source && p->uid)
+		if (p->source && p->uid && camel_folder_has_summary_capability (p->source))
 			camel_folder_set_message_user_tag(p->source, p->uid, "colour", argv[0]->value.string);
 		else
 			camel_tag_set (&p->info->user_tags, "colour", argv[0]->value.string);
@@ -476,7 +589,7 @@ do_score (struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFilterDri
 		char *value;
 		
 		value = g_strdup_printf ("%d", argv[0]->value.number);
-		if (p->source && p->uid)
+		if (p->source && p->uid && camel_folder_has_summary_capability (p->source))
 			camel_folder_set_message_user_tag(p->source, p->uid, "score", value);
 		else
 			camel_tag_set (&p->info->user_tags, "score", value);
@@ -488,22 +601,128 @@ do_score (struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFilterDri
 }
 
 static ESExpResult *
-do_flag (struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFilterDriver *driver)
+set_flag (struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFilterDriver *driver)
 {
 	struct _CamelFilterDriverPrivate *p = _PRIVATE (driver);
+	guint32 flags;
 	
 	d(fprintf (stderr, "setting flag\n"));
 	if (argc == 1 && argv[0]->type == ESEXP_RES_STRING) {
-		if (p->source && p->uid)
-			camel_folder_set_message_flags(p->source, p->uid, camel_system_flag(argv[0]->value.string), ~0);
+		flags = camel_system_flag (argv[0]->value.string);
+		if (p->source && p->uid && camel_folder_has_summary_capability (p->source))
+			camel_folder_set_message_flags (p->source, p->uid, flags, ~0);
 		else
-			p->info->flags |= camel_system_flag (argv[0]->value.string)|CAMEL_MESSAGE_FOLDER_FLAGGED;
+			p->info->flags |= flags | CAMEL_MESSAGE_FOLDER_FLAGGED;
 		camel_filter_driver_log (driver, FILTER_LOG_ACTION, "Set %s flag", argv[0]->value.string);
 	}
 	
 	return NULL;
 }
 
+static ESExpResult *
+unset_flag (struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFilterDriver *driver)
+{
+	struct _CamelFilterDriverPrivate *p = _PRIVATE (driver);
+	guint32 flags;
+	
+	d(fprintf (stderr, "unsetting flag\n"));
+	if (argc == 1 && argv[0]->type == ESEXP_RES_STRING) {
+		flags = camel_system_flag (argv[0]->value.string);
+		if (p->source && p->uid && camel_folder_has_summary_capability (p->source))
+			camel_folder_set_message_flags (p->source, p->uid, flags, 0);
+		else
+			p->info->flags = (p->info->flags & ~flags) | CAMEL_MESSAGE_FOLDER_FLAGGED;
+		camel_filter_driver_log (driver, FILTER_LOG_ACTION, "Unset %s flag", argv[0]->value.string);
+	}
+	
+	return NULL;
+}
+
+static ESExpResult *
+do_shell (struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFilterDriver *driver)
+{
+	struct _CamelFilterDriverPrivate *p = _PRIVATE (driver);
+	GString *command;
+	GPtrArray *args;
+	int i;
+	
+	d(fprintf (stderr, "executing shell command\n"));
+	
+	command = g_string_new ("");
+	
+	args = g_ptr_array_new ();
+	
+	/* make sure all args are strings */
+	for (i = 0; i < argc; i++) {
+		if (argv[i]->type != ESEXP_RES_STRING)
+			goto done;
+		
+		g_ptr_array_add (args, argv[i]->value.string);
+		
+		g_string_append (command, argv[i]->value.string);
+		g_string_append_c (command, ' ');
+	}
+	
+	g_string_truncate (command, command->len - 1);
+	
+	if (p->shellfunc && argc >= 1) {
+		p->shellfunc (driver, argc, (char **) args->pdata, p->shelldata);
+		camel_filter_driver_log (driver, FILTER_LOG_ACTION, "Executing shell command: [%s]",
+					 command->str);
+	}
+	
+ done:
+	
+	g_ptr_array_free (args, TRUE);
+	g_string_free (command, TRUE);
+	
+	return NULL;
+}
+
+static ESExpResult *
+do_beep (struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFilterDriver *driver)
+{
+	struct _CamelFilterDriverPrivate *p = _PRIVATE (driver);
+	
+	d(fprintf (stderr, "beep\n"));
+	
+	if (p->beep) {
+		p->beep (driver, p->beepdata);
+		camel_filter_driver_log (driver, FILTER_LOG_ACTION, "Beep");
+	}
+	
+	return NULL;
+}
+
+static ESExpResult *
+play_sound (struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFilterDriver *driver)
+{
+	struct _CamelFilterDriverPrivate *p = _PRIVATE (driver);
+	
+	d(fprintf (stderr, "play sound\n"));
+	
+	if (p->playfunc && argc == 1 && argv[0]->type == ESEXP_RES_STRING) {
+		p->playfunc (driver, argv[0]->value.string, p->playdata);
+		camel_filter_driver_log (driver, FILTER_LOG_ACTION, "Play sound");
+	}
+	
+	return NULL;
+}
+
+static ESExpResult *
+do_only_once (struct _ESExp *f, int argc, struct _ESExpResult **argv, CamelFilterDriver *driver)
+{
+	struct _CamelFilterDriverPrivate *p = _PRIVATE (driver);
+	
+	d(fprintf (stderr, "only once\n"));
+	
+	if (argc == 2 && !g_hash_table_lookup (p->only_once, argv[0]->value.string))
+		g_hash_table_insert (p->only_once, g_strdup (argv[0]->value.string),
+				     g_strdup (argv[1]->value.string));
+	
+	return NULL;
+}
+
 static CamelFolder *
 open_folder (CamelFilterDriver *driver, const char *folder_url)
 {
@@ -594,22 +813,20 @@ camel_filter_driver_log (CamelFilterDriver *driver, enum filter_log_t status, co
 		case FILTER_LOG_START: {
 			/* write log header */
 			const char *subject = NULL;
-			char *fromstr;
-			const CamelInternetAddress *from;
+			const char *from = NULL;
 			char date[50];
 			time_t t;
 			
 			/* FIXME: does this need locking?  Probably */
 			
-			from = camel_mime_message_get_from (p->message);
-			fromstr = camel_address_format((CamelAddress *)from);
-			subject = camel_mime_message_get_subject (p->message);
+			from = camel_message_info_from (p->info);
+			subject = camel_message_info_subject (p->info);
 			
 			time (&t);
 			strftime (date, 49, "%a, %d %b %Y %H:%M:%S", localtime (&t));
 			fprintf (p->logfile, "Applied filter \"%s\" to message from %s - \"%s\" at %s\n",
-				 str, fromstr ? fromstr : "unknown", subject ? subject : "", date);
-			g_free(fromstr);
+				 str, from ? from : "unknown", subject ? subject : "", date);
+			
 			break;
 		}
 		case FILTER_LOG_ACTION:
@@ -628,6 +845,71 @@ camel_filter_driver_log (CamelFilterDriver *driver, enum filter_log_t status, co
 }
 
 
+struct _run_only_once {
+	CamelFilterDriver *driver;
+	CamelException *ex;
+};
+
+static gboolean
+run_only_once (gpointer key, char *action, struct _run_only_once *data)
+{
+	struct _CamelFilterDriverPrivate *p = _PRIVATE (data->driver);
+	CamelException *ex = data->ex;
+	ESExpResult *r;
+	
+	printf ("evaluating: %s\n\n", action);
+	
+	e_sexp_input_text (p->eval, action, strlen (action));
+	if (e_sexp_parse (p->eval) == -1) {
+		if (!camel_exception_is_set (ex))
+			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
+					      _("Error parsing filter: %s: %s"),
+					      e_sexp_error (p->eval), action);
+		goto done;
+	}
+	
+	r = e_sexp_eval (p->eval);
+	if (r == NULL) {
+		if (!camel_exception_is_set (ex))
+			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
+					      _("Error executing filter: %s: %s"),
+					      e_sexp_error (p->eval), action);
+		goto done;
+	}
+	
+	e_sexp_result_free (p->eval, r);
+	
+ done:
+	
+	g_free (key);
+	g_free (action);
+	
+	return TRUE;
+}
+
+
+/**
+ * camel_filter_driver_flush:
+ * @driver:
+ * @ex:
+ *
+ * Flush all of the only-once filter actions.
+ **/
+void
+camel_filter_driver_flush (CamelFilterDriver *driver, CamelException *ex)
+{
+	struct _CamelFilterDriverPrivate *p = _PRIVATE (driver);
+	struct _run_only_once data;
+	
+	if (!p->only_once)
+		return;
+	
+	data.driver = driver;
+	data.ex = ex;
+	
+	g_hash_table_foreach_remove (p->only_once, (GHRFunc) run_only_once, &data);
+}
+
 /**
  * camel_filter_driver_filter_mbox:
  * @driver: CamelFilterDriver
@@ -653,6 +935,7 @@ camel_filter_driver_filter_mbox (CamelFilterDriver *driver, const char *mbox, co
 	struct stat st;
 	int status;
 	off_t last = 0;
+	int ret = -1;
 	
 	fd = open (mbox, O_RDONLY);
 	if (fd == -1) {
@@ -684,8 +967,8 @@ camel_filter_driver_filter_mbox (CamelFilterDriver *driver, const char *mbox, co
 		
 		msg = camel_mime_message_new ();
 		if (camel_mime_part_construct_from_parser (CAMEL_MIME_PART (msg), mp) == -1) {
+			camel_exception_set (ex, (errno==EINTR)?CAMEL_EXCEPTION_USER_CANCEL:CAMEL_EXCEPTION_SYSTEM, _("Cannot open message"));
 			report_status (driver, CAMEL_FILTER_STATUS_END, 100, _("Failed on message %d"), i);
-			camel_exception_set (ex, CAMEL_EXCEPTION_SYSTEM, _("Cannot open message"));
 			camel_object_unref (CAMEL_OBJECT (msg));
 			goto fail;
 		}
@@ -712,14 +995,12 @@ camel_filter_driver_filter_mbox (CamelFilterDriver *driver, const char *mbox, co
 	
 	if (p->defaultfolder) {
 		report_status(driver, CAMEL_FILTER_STATUS_PROGRESS, 100, _("Syncing folder"));
-		camel_folder_sync(p->defaultfolder, FALSE, ex);
+		camel_folder_sync(p->defaultfolder, FALSE, camel_exception_is_set (ex) ? NULL : ex);
 	}
 	
 	report_status (driver, CAMEL_FILTER_STATUS_END, 100, _("Complete"));
-	
-	g_free (source_url);
-	return 0;
-	
+
+	ret = 0;
 fail:
 	g_free (source_url);
 	if (fd != -1)
@@ -752,28 +1033,32 @@ camel_filter_driver_filter_folder (CamelFilterDriver *driver, CamelFolder *folde
 				   GPtrArray *uids, gboolean remove, CamelException *ex)
 {
 	struct _CamelFilterDriverPrivate *p = _PRIVATE (driver);
-	int i;
-	int freeuids = FALSE;
-	CamelMimeMessage *message;
+	gboolean freeuids = FALSE;
 	CamelMessageInfo *info;
 	char *source_url, *service_url;
 	const char *folder_name;
 	int status = 0;
-	int need_sep = 0;
+	CamelURL *url;
+	int i;
 	
 	service_url = camel_service_get_url (CAMEL_SERVICE (camel_folder_get_parent_store (folder)));
+	url = camel_url_new (service_url, NULL);
+	g_free (service_url);
+	
 	folder_name = camel_folder_get_full_name (folder);
 	
-	/* Add a separator unless the first char of folder_name or the last char of service_url is '/' */
-	need_sep = (folder_name && *folder_name != '/');
-	if (service_url && *service_url && !need_sep) {
-		need_sep = (service_url[strlen (service_url)-1] != '/');
+	if (folder_name && *folder_name != '/') {
+		char *path;
+		
+		path = g_strdup_printf ("/%s", folder_name);
+		camel_url_set_path (url, path);
+		g_free (path);
+	} else {
+		camel_url_set_path (url, folder_name);
 	}
-	source_url = g_strdup_printf ("%s%s%s",
-				      service_url,
-				      need_sep ? "/" : "",
-				      folder_name);
-	g_free (service_url);
+	
+	source_url = camel_url_to_string (url, CAMEL_URL_HIDE_ALL);
+	camel_url_free (url);
 	
 	if (uids == NULL) {
 		uids = camel_folder_get_uids (folder);
@@ -786,20 +1071,12 @@ camel_filter_driver_filter_folder (CamelFilterDriver *driver, CamelFolder *folde
 		report_status (driver, CAMEL_FILTER_STATUS_START, pc, _("Getting message %d of %d"), i+1,
 			       uids->len);
 		
-		message = camel_folder_get_message (folder, uids->pdata[i], ex);
-		if (!message || camel_exception_is_set (ex)) {
-			report_status (driver, CAMEL_FILTER_STATUS_END, 100, _("Failed at message %d of %d"),
-				       i+1, uids->len);
-			status = -1;
-			break;
-		}
-		
 		if (camel_folder_has_summary_capability (folder))
 			info = camel_folder_get_message_info (folder, uids->pdata[i]);
 		else
 			info = NULL;
 		
-		status = camel_filter_driver_filter_message (driver, message, info, uids->pdata[i],
+		status = camel_filter_driver_filter_message (driver, NULL, info, uids->pdata[i],
 							     folder, source_url, source_url, ex);
 		
 		if (camel_folder_has_summary_capability (folder))
@@ -818,8 +1095,6 @@ camel_filter_driver_filter_folder (CamelFilterDriver *driver, CamelFolder *folde
 		
 		if (cache)
 			camel_uid_cache_save_uid (cache, uids->pdata[i]);
-		
-		camel_object_unref (CAMEL_OBJECT (message));
 	}
 	
 	if (freeuids)
@@ -834,15 +1109,49 @@ camel_filter_driver_filter_folder (CamelFilterDriver *driver, CamelFolder *folde
 		report_status (driver, CAMEL_FILTER_STATUS_END, 100, _("Complete"));
 	
 	g_free (source_url);
-
+	
 	return status;
 }
 
 
+struct _get_message {
+	struct _CamelFilterDriverPrivate *p;
+	const char *source_url;
+};
+
+
+static CamelMimeMessage *
+get_message_cb (void *data, CamelException *ex)
+{
+	struct _get_message *msgdata = data;
+	struct _CamelFilterDriverPrivate *p = msgdata->p;
+	const char *source_url = msgdata->source_url;
+	CamelMimeMessage *message;
+	
+	if (p->message) {
+		message = p->message;
+		camel_object_ref (CAMEL_OBJECT (message));
+	} else {
+		const char *uid;
+		
+		if (p->uid)
+			uid = p->uid;
+		else
+			uid = camel_message_info_uid (p->info);
+		
+		message = camel_folder_get_message (p->source, uid, ex);
+	}
+	
+	if (source_url && message && camel_mime_message_get_source (message) == NULL)
+		camel_mime_message_set_source (message, source_url);
+	
+	return message;
+}
+
 /**
  * camel_filter_driver_filter_message:
  * @driver: CamelFilterDriver
- * @message: message to filter
+ * @message: message to filter or NULL
  * @info: message info or NULL
  * @uid: message uid or NULL
  * @source: source folder or NULL
@@ -874,14 +1183,34 @@ camel_filter_driver_filter_message (CamelFilterDriver *driver, CamelMimeMessage
 	ESExpResult *r;
 	int result;
 	
+	/* FIXME: make me into a g_return_if_fail/g_assert or whatever... */
+	if (message == NULL && (source == NULL || uid == NULL)) {
+		g_warning ("there is no way to fetch the message using the information provided...");
+		return -1;
+	}
+	
 	if (info == NULL) {
-		struct _header_raw *h = CAMEL_MIME_PART (message)->headers;
+		struct _header_raw *h;
 		
+		if (message) {
+			camel_object_ref (CAMEL_OBJECT (message));
+		} else {
+			message = camel_folder_get_message (source, uid, ex);
+			if (!message)
+				return -1;
+		}
+		
+		h = CAMEL_MIME_PART (message)->headers;
 		info = camel_message_info_new_from_header (h);
 		freeinfo = TRUE;
 	} else {
 		if (info->flags & CAMEL_MESSAGE_DELETED)
 			return 0;
+		
+		uid = camel_message_info_uid (info);
+		
+		if (message)
+			camel_object_ref (CAMEL_OBJECT (message));
 	}
 	
 	p->ex = ex;
@@ -893,18 +1222,23 @@ camel_filter_driver_filter_message (CamelFilterDriver *driver, CamelMimeMessage
 	p->uid = uid;
 	p->source = source;
 	
-	if (original_source_url && camel_mime_message_get_source (message) == NULL)
+	if (message && original_source_url && camel_mime_message_get_source (message) == NULL)
 		camel_mime_message_set_source (message, original_source_url);
 	
-	node = (struct _filter_rule *)p->rules.head;
+	node = (struct _filter_rule *) p->rules.head;
 	result = CAMEL_SEARCH_NOMATCH;
 	while (node->next && !p->terminated) {
+		struct _get_message data;
+		
 		d(fprintf (stderr, "applying rule %s\naction %s\n", node->match, node->action));
 		
-		result = camel_filter_search_match (p->message, p->info, 
+		data.p = p;
+		data.source_url = original_source_url;
+		
+		result = camel_filter_search_match (p->session, get_message_cb, &data, p->info, 
 						    original_source_url ? original_source_url : source_url,
 						    node->match, p->ex);
-
+		
 		switch (result) {
 		case CAMEL_SEARCH_ERROR:
 			goto error;
@@ -935,7 +1269,7 @@ camel_filter_driver_filter_message (CamelFilterDriver *driver, CamelMimeMessage
 	
 	/* *Now* we can set the DELETED flag... */
 	if (p->deleted) {
-		if (p->source && p->uid)
+		if (p->source && p->uid && camel_folder_has_summary_capability (p->source))
 			camel_folder_set_message_flags(p->source, p->uid, CAMEL_MESSAGE_DELETED|CAMEL_MESSAGE_SEEN, ~0);
 		else
 			info->flags |= CAMEL_MESSAGE_DELETED|CAMEL_MESSAGE_SEEN|CAMEL_MESSAGE_FOLDER_FLAGGED;
@@ -951,22 +1285,34 @@ camel_filter_driver_filter_message (CamelFilterDriver *driver, CamelMimeMessage
 			
 			uids = g_ptr_array_new ();
 			g_ptr_array_add (uids, (char *) p->uid);
-			camel_folder_copy_messages_to (p->source, uids, p->defaultfolder, p->ex);
+			camel_folder_transfer_messages_to (p->source, uids, p->defaultfolder, NULL, FALSE, p->ex);
 			g_ptr_array_free (uids, TRUE);
 		} else {
-			camel_folder_append_message (p->defaultfolder, p->message, p->info, p->ex);
+			if (p->message == NULL) {
+				p->message = camel_folder_get_message (source, uid, ex);
+				if (!p->message)
+					goto error;
+			}
+			
+			camel_folder_append_message (p->defaultfolder, p->message, p->info, NULL, p->ex);
 		}
 	}
 	
+	if (p->message)
+		camel_object_unref (CAMEL_OBJECT (p->message));
+	
 	if (freeinfo)
 		camel_message_info_free (info);
 	
 	return 0;
 	
-error:	
+ error:
 	if (filtered)
 		camel_filter_driver_log (driver, FILTER_LOG_END, NULL);
 	
+	if (p->message)
+		camel_object_unref (CAMEL_OBJECT (p->message));
+	
 	if (freeinfo)
 		camel_message_info_free (info);
 	
