@@ -43,6 +43,7 @@
 #include "camel-mime-filter-charset.h"
 #include "camel-mime-part.h"
 #include "camel-mime-part-utils.h"
+#include "camel-mime-utils.h"
 #include "camel-exception.h"
 #include "camel-charset-map.h"
 #include "string-utils.h"
@@ -92,6 +93,11 @@ static int             construct_from_parser           (CamelMimePart *, CamelMi
 /* forward references */
 static void set_disposition (CamelMimePart *mime_part, const gchar *disposition);
 
+/* format output of headers */
+static int write_references(CamelStream *stream, struct _header_raw *h);
+/*static int write_fold(CamelStream *stream, struct _header_raw *h);*/
+static int write_raw(CamelStream *stream, struct _header_raw *h);
+
 
 /* loads in a hash table the set of header names we */
 /* recognize and associate them with a unique enum  */
@@ -109,14 +115,16 @@ init_header_name_table()
 	g_hash_table_insert (header_name_table, "Content-Type", (gpointer)HEADER_CONTENT_TYPE);
 
 	header_formatted_table = g_hash_table_new(g_strcase_hash, g_strcase_equal);
-	g_hash_table_insert(header_formatted_table, "Content-Type", (void *)1);
-	g_hash_table_insert(header_formatted_table, "Content-Disposition", (void *)1);
-	g_hash_table_insert(header_formatted_table, "To", (void *)1);
-	g_hash_table_insert(header_formatted_table, "From", (void *)1);
-	g_hash_table_insert(header_formatted_table, "Reply-To", (void *)1);
-	g_hash_table_insert(header_formatted_table, "Cc", (void *)1);
-	g_hash_table_insert(header_formatted_table, "Bcc", (void *)1);
-	g_hash_table_insert(header_formatted_table, "Message-ID", (void *)1);
+	g_hash_table_insert(header_formatted_table, "Content-Type", write_raw);
+	g_hash_table_insert(header_formatted_table, "Content-Disposition", write_raw);
+	g_hash_table_insert(header_formatted_table, "To", write_raw);
+	g_hash_table_insert(header_formatted_table, "From", write_raw);
+	g_hash_table_insert(header_formatted_table, "Reply-To", write_raw);
+	g_hash_table_insert(header_formatted_table, "Cc", write_raw);
+	g_hash_table_insert(header_formatted_table, "Bcc", write_raw);
+	g_hash_table_insert(header_formatted_table, "Message-ID", write_raw);
+	g_hash_table_insert(header_formatted_table, "In-Reply-To", write_raw);
+	g_hash_table_insert(header_formatted_table, "References", write_references);
 }
 
 static void
@@ -163,7 +171,7 @@ static void
 camel_mime_part_finalize (CamelObject *object)
 {
 	CamelMimePart *mime_part = CAMEL_MIME_PART (object);
-
+	
 	g_free (mime_part->description);
 	g_free (mime_part->content_id);
 	g_free (mime_part->content_MD5);
@@ -205,7 +213,7 @@ process_header(CamelMedium *medium, const char *header_name, const char *header_
 {
 	CamelMimePart *mime_part = CAMEL_MIME_PART (medium);
 	CamelHeaderType header_type;
-	const char *charset;
+	const char *charset, *p;
 	char *text;
 
 	/* Try to parse the header pair. If it corresponds to something   */
@@ -227,7 +235,21 @@ process_header(CamelMedium *medium, const char *header_name, const char *header_
 		break;
 	case HEADER_CONTENT_ID:
 		g_free (mime_part->content_id);
-		mime_part->content_id = header_msgid_decode (header_value);
+		if (!(mime_part->content_id = header_msgid_decode (header_value))) {
+			while (*header_value && strchr (" \t\r\n", *header_value))
+				header_value++;
+			if (*header_value == '<') {
+				p = header_value;
+				while (*p && *p != '>')
+					p++;
+				mime_part->content_id = g_strndup (header_value, p - header_value);
+			} else if (*header_value) {
+				mime_part->content_id = g_strdup (header_value);
+			}
+			
+			if (mime_part->content_id)
+				g_strstrip (mime_part->content_id);
+		}
 		break;
 	case HEADER_ENCODING:
 		text = header_token_decode (header_value);
@@ -242,7 +264,7 @@ process_header(CamelMedium *medium, const char *header_name, const char *header_
 		g_free (mime_part->content_location);
 		mime_part->content_location = header_location_decode (header_value);
 		break;
-	case HEADER_CONTENT_TYPE: 
+	case HEADER_CONTENT_TYPE:
 		if (mime_part->content_type)
 			header_content_type_unref (mime_part->content_type);
 		mime_part->content_type = header_content_type_decode (header_value);
@@ -386,6 +408,7 @@ void
 camel_mime_part_set_filename (CamelMimePart *mime_part, const gchar *filename)
 {
 	char *str;
+	
 	if (mime_part->disposition == NULL)
 		mime_part->disposition = header_disposition_decode("attachment");
 
@@ -395,6 +418,11 @@ camel_mime_part_set_filename (CamelMimePart *mime_part, const gchar *filename)
 	camel_medium_set_header (CAMEL_MEDIUM (mime_part),
 				 "Content-Disposition", str);
 	g_free(str);
+	
+	header_content_type_set_param (mime_part->content_type, "name", filename);
+	str = header_content_type_format (mime_part->content_type);
+	camel_medium_set_header (CAMEL_MEDIUM (mime_part), "Content-Type", str);
+	g_free (str);
 }
 
 const gchar *
@@ -540,11 +568,78 @@ set_content_object (CamelMedium *medium, CamelDataWrapper *content)
 
 /**********************************************************************/
 
+static int
+write_references(CamelStream *stream, struct _header_raw *h)
+{
+	int len, out, total;
+	char *v, *ids, *ide;
+
+	/* this is only approximate, based on the next >, this way it retains any content
+	   from the original which may not be properly formatted, etc.  It also doesn't handle
+	   the case where an individual messageid is too long, however thats a bad mail to
+	   start with ... */
+
+	v = h->value;
+	len = strlen(h->name)+1;
+	total = camel_stream_printf(stream, "%s%s", h->name, isspace(v[0])?":":": ");
+	if (total == -1)
+		return -1;
+	while (*v) {
+		ids = v;
+		ide = strchr(ids+1, '>');
+		if (ide)
+			v = ++ide;
+		else
+			ide = v = strlen(ids)+ids;
+
+		if (len>0 && len + (ide - ids) >= CAMEL_FOLD_SIZE) {
+			out = camel_stream_printf(stream, "\n\t");
+			if (out == -1)
+				return -1;
+			total += out;
+			len = 0;
+		}
+		out = camel_stream_write(stream, ids, ide-ids);
+		if (out == -1)
+			return -1;
+		len += out;
+		total += out;
+	}
+	camel_stream_write(stream, "\n", 1);
+
+	return total;
+}
+
+#if 0
+/* not needed - yet - handled by default case */
+static int
+write_fold(CamelStream *stream, struct _header_raw *h)
+{
+	char *val;
+	int count;
+
+	val = header_fold(h->value, strlen(h->name));
+	count = camel_stream_printf(stream, "%s%s%s\n", h->name, isspace(val[0]) ? ":" : ": ", val);
+	g_free(val);
+
+	return count;
+}
+#endif
+
+static int
+write_raw(CamelStream *stream, struct _header_raw *h)
+{
+	char *val = h->value;
+
+	return camel_stream_printf(stream, "%s%s%s\n", h->name, isspace(val[0]) ? ":" : ": ", val);
+}
+
 static int
 write_to_stream(CamelDataWrapper *data_wrapper, CamelStream *stream)
 {
 	CamelMimePart *mp = CAMEL_MIME_PART(data_wrapper);
 	CamelMedium *medium = CAMEL_MEDIUM(data_wrapper);
+	CamelStream *ostream = stream;
 	CamelDataWrapper *content;
 	int total = 0;
 	int count;
@@ -556,11 +651,12 @@ write_to_stream(CamelDataWrapper *data_wrapper, CamelStream *stream)
 #ifndef NO_WARNINGS
 #warning content-languages should be stored as a header
 #endif
-
+	
 	if (mp->headers) {
 		struct _header_raw *h = mp->headers;
 		char *val;
-
+		int (*writefn)(CamelStream *stream, struct _header_raw *);
+		
 		/* fold/write the headers.   But dont fold headers that are already formatted
 		   (e.g. ones with parameter-lists, that we know about, and have created) */
 		while (h) {
@@ -568,12 +664,12 @@ write_to_stream(CamelDataWrapper *data_wrapper, CamelStream *stream)
 			if (val == NULL) {
 				g_warning("h->value is NULL here for %s", h->name);
 				count = 0;
-			} else if (g_hash_table_lookup(header_formatted_table, h->name) == NULL) {
+			} else if ((writefn = g_hash_table_lookup(header_formatted_table, h->name)) == NULL) {
 				val = header_fold(val, strlen(h->name));
 				count = camel_stream_printf(stream, "%s%s%s\n", h->name, isspace(val[0]) ? ":" : ": ", val);
 				g_free(val);
 			} else {
-				count = camel_stream_printf(stream, "%s%s%s\n", h->name, isspace(val[0]) ? ":" : ": ", val);
+				count = writefn(stream, h);
 			}
 			if (count == -1)
 				return -1;
@@ -581,12 +677,12 @@ write_to_stream(CamelDataWrapper *data_wrapper, CamelStream *stream)
 			h = h->next;
 		}
 	}
-
+	
 	count = camel_stream_write(stream, "\n", 1);
 	if (count == -1)
 		return -1;
 	total += count;
-
+	
 	content = camel_medium_get_content_object(medium);
 	if (content) {
 		/* I dont really like this here, but i dont know where else it might go ... */
@@ -595,20 +691,29 @@ write_to_stream(CamelDataWrapper *data_wrapper, CamelStream *stream)
 		CamelMimeFilter *filter = NULL;
 		CamelStreamFilter *filter_stream = NULL;
 		CamelMimeFilter *charenc = NULL;
+		const char *filename;
 		const char *charset;
-
-		switch(mp->encoding) {
+		
+		switch (mp->encoding) {
 		case CAMEL_MIME_PART_ENCODING_BASE64:
 			filter = (CamelMimeFilter *)camel_mime_filter_basic_new_type(CAMEL_MIME_FILTER_BASIC_BASE64_ENC);
 			break;
 		case CAMEL_MIME_PART_ENCODING_QUOTEDPRINTABLE:
 			filter = (CamelMimeFilter *)camel_mime_filter_basic_new_type(CAMEL_MIME_FILTER_BASIC_QP_ENC);
 			break;
+		case CAMEL_MIME_PART_ENCODING_UUENCODE:
+			filename = camel_mime_part_get_filename (mp);
+			count = camel_stream_printf (ostream, "begin 644 %s\n", filename ? filename : "untitled");
+			if (count == -1)
+				return -1;
+			total += count;
+			filter = (CamelMimeFilter *)camel_mime_filter_basic_new_type(CAMEL_MIME_FILTER_BASIC_UU_ENC);
+			break;
 		default:
 			break;
 		}
 		
-		if (!data_wrapper->rawtext && header_content_type_is(mp->content_type, "text", "*")) {
+		if (!content->rawtext && header_content_type_is(mp->content_type, "text", "*")) {
 			charset = header_content_type_param(mp->content_type, "charset");
 			if (charset && !(!strcasecmp(charset, "us-ascii") || !strcasecmp(charset, "utf-8"))) {
 				charenc = (CamelMimeFilter *)camel_mime_filter_charset_new_convert("UTF-8", charset);
@@ -617,33 +722,34 @@ write_to_stream(CamelDataWrapper *data_wrapper, CamelStream *stream)
 		
 		if (filter || charenc) {
 			filter_stream = camel_stream_filter_new_with_stream(stream);
-
+			
 			/* if we have a character encoder, add that always */
 			if (charenc) {
 				camel_stream_filter_add(filter_stream, charenc);
 				camel_object_unref((CamelObject *)charenc);
 			}
-
+			
 			/* we only re-do crlf on encoded blocks */
 			if (filter && header_content_type_is(mp->content_type, "text", "*")) {
 				CamelMimeFilter *crlf = camel_mime_filter_crlf_new(CAMEL_MIME_FILTER_CRLF_ENCODE,
 										   CAMEL_MIME_FILTER_CRLF_MODE_CRLF_ONLY);
-
+				
 				camel_stream_filter_add(filter_stream, crlf);
 				camel_object_unref((CamelObject *)crlf);
-
 			}
-
+			
 			if (filter) {
 				camel_stream_filter_add(filter_stream, filter);
 				camel_object_unref((CamelObject *)filter);
 			}
-
+			
 			stream = (CamelStream *)filter_stream;
 		}
 
 #endif
+		
 		count = camel_data_wrapper_write_to_stream(content, stream);
+		
 		if (filter_stream) {
 			camel_stream_flush((CamelStream *)filter_stream);
 			camel_object_unref((CamelObject *)filter_stream);
@@ -651,9 +757,17 @@ write_to_stream(CamelDataWrapper *data_wrapper, CamelStream *stream)
 		if (count == -1)
 			return -1;
 		total += count;
+		
+		if (mp->encoding == CAMEL_MIME_PART_ENCODING_UUENCODE) {
+			count = camel_stream_write (ostream, "end\n", 4);
+			if (count == -1)
+				return -1;
+			total += count;
+		}
 	} else {
 		g_warning("No content for medium, nothing to write");
 	}
+	
 	return total;
 }
 
@@ -686,9 +800,14 @@ construct_from_parser(CamelMimePart *dw, CamelMimeParser *mp)
 			process_header((CamelMedium *)dw, "content-type", content);
 
 		while (headers) {
-			camel_medium_add_header((CamelMedium *)dw, headers->name, headers->value);
+			if (strcasecmp(headers->name, "content-type") == 0
+			    && headers->value != content)
+				camel_medium_add_header((CamelMedium *)dw, "X-Invalid-Content-Type", headers->value);
+			else
+				camel_medium_add_header((CamelMedium *)dw, headers->name, headers->value);
 			headers = headers->next;
 		}
+
 		camel_mime_part_construct_content_from_parser(dw, mp);
 		break;
 	default:
@@ -750,7 +869,8 @@ static const char *encodings[] = {
 	"8bit",
 	"base64",
 	"quoted-printable",
-	"binary"
+	"binary",
+	"x-uuencode",
 };
 
 const char *
