@@ -236,7 +236,7 @@ identity_changed (GtkWidget *widget, gpointer data)
 	
 	if (gui->page != MAIL_CONFIG_WIZARD_PAGE_IDENTITY)
 		return;
-
+	
 	next_sensitive = mail_account_gui_identity_complete (gui->gui, &incomplete);
 	
 	evolution_wizard_set_buttons_sensitive (gui->wizard, TRUE, next_sensitive, TRUE, NULL);
@@ -783,6 +783,8 @@ get_fn (EvolutionWizard *wizard,
                                     "changed", identity_changed, gui);
                 gtk_signal_connect (GTK_OBJECT (gui->gui->email_address),
                                     "changed", identity_changed, gui);
+		gtk_signal_connect (GTK_OBJECT (gui->gui->reply_to),
+                                    "changed", identity_changed, gui);
 		gtk_signal_connect (GTK_OBJECT (gui->gui->source.hostname), 
 				    "changed", source_changed, gui);
 		gtk_signal_connect (GTK_OBJECT (gui->gui->source.username),
@@ -941,7 +943,9 @@ wizard_back_cb (EvolutionWizard *wizard,
                 int page_num,
                 MailConfigWizard *gui) 
 {
-        if (wizard_pages[page_num].back_func == NULL
+	if (page_num >= 5)
+		evolution_wizard_set_page(wizard, 4, NULL);
+        else if (wizard_pages[page_num].back_func == NULL
 	    || !(wizard_pages[page_num].back_func (wizard, gui))) {
 		if (page_num > 0)
 			evolution_wizard_set_page(wizard, page_num-1, NULL);
