@@ -47,22 +47,6 @@
 
 /* **************************************** */
 
-gchar *
-mail_tool_get_local_movemail_path (void)
-{
-	static gint count = 0;
-	static pthread_mutex_t movemail_path_lock = PTHREAD_MUTEX_INITIALIZER;
-	gint my_count;
-
-	/* Ah, the joys of being multi-threaded... */
-	pthread_mutex_lock (&movemail_path_lock);
-	my_count = count;
-	++count;
-	pthread_mutex_unlock (&movemail_path_lock);
-
-	return g_strdup_printf ("%s/local/Inbox/movemail.%d", evolution_dir, my_count);
-}
-
 CamelFolder *
 mail_tool_get_local_inbox (CamelException *ex)
 {
@@ -115,18 +99,35 @@ mail_tool_get_trash (const gchar *url, int connect, CamelException *ex)
 	return trash;
 }
 
+static char *
+mail_tool_get_local_movemail_path (const unsigned char *uri)
+{
+	unsigned char *safe_uri, *c;
+	char *path;
+	
+	safe_uri = g_strdup (uri);
+	for (c = safe_uri; *c; c++)
+		if (strchr ("/:;=|%&#!*^()\\, ", *c) || !isprint ((int) *c))
+			*c = '_';
+	
+	path = g_strdup_printf ("%s/local/Inbox/movemail.%s", evolution_dir, safe_uri);
+	g_free (safe_uri);
+	
+	return path;
+}
+
 /* why is this function so stupidly complex when allthe work is done elsehwere? */
 char *
-mail_tool_do_movemail (const gchar *source_url, CamelException *ex)
+mail_tool_do_movemail (const char *source_url, CamelException *ex)
 {
-	gchar *dest_path;
-	const gchar *source;
+	char *dest_path;
+	const char *source;
 	struct stat sb;
-
+	
 	g_return_val_if_fail (strncmp (source_url, "mbox:", 5) == 0, NULL);
 	
 	/* Set up our destination. */
-	dest_path = mail_tool_get_local_movemail_path();
+	dest_path = mail_tool_get_local_movemail_path (source_url);
 	
 	/* Skip over "mbox:" plus host part (if any) of url. */
 	source = source_url + 5;
@@ -200,6 +201,7 @@ mail_tool_remove_xevolution_headers (CamelMimeMessage *message)
 	xev->account = g_strdup (camel_medium_get_header (CAMEL_MEDIUM (message), "X-Evolution-Account"));
 	xev->fcc = g_strdup (camel_medium_get_header (CAMEL_MEDIUM (message), "X-Evolution-Fcc"));
 	xev->format = g_strdup (camel_medium_get_header (CAMEL_MEDIUM (message), "X-Evolution-Format"));
+	xev->postto = g_strdup (camel_medium_get_header (CAMEL_MEDIUM (message), "X-Evolution-PostTo"));
 	
 	/* rip off the X-Evolution* headers */
 	camel_medium_remove_header (CAMEL_MEDIUM (message), "X-Evolution");
@@ -208,6 +210,7 @@ mail_tool_remove_xevolution_headers (CamelMimeMessage *message)
 	camel_medium_remove_header (CAMEL_MEDIUM (message), "X-Evolution-Account");
 	camel_medium_remove_header (CAMEL_MEDIUM (message), "X-Evolution-Fcc");
 	camel_medium_remove_header (CAMEL_MEDIUM (message), "X-Evolution-Format");
+	camel_medium_remove_header (CAMEL_MEDIUM (message), "X-Evolution-PostTo");
 	
 	return xev;
 }
@@ -227,6 +230,8 @@ mail_tool_restore_xevolution_headers (CamelMimeMessage *message, XEvolution *xev
 		camel_medium_set_header (CAMEL_MEDIUM (message), "X-Evolution-Fcc", xev->fcc);
 	if (xev->format)
 		camel_medium_set_header (CAMEL_MEDIUM (message), "X-Evolution-Format", xev->format);
+	if (xev->postto)
+		camel_medium_set_header (CAMEL_MEDIUM (message), "X-Evolution-PostTo", xev->postto);
 }
 
 void
@@ -236,7 +241,9 @@ mail_tool_destroy_xevolution (XEvolution *xev)
 	g_free (xev->source);
 	g_free (xev->transport);
 	g_free (xev->account);
+	g_free (xev->format);
 	g_free (xev->fcc);
+	g_free (xev->postto);
 	g_free (xev);
 }
 
@@ -263,6 +270,10 @@ mail_tool_make_message_attachment (CamelMimeMessage *message)
 	xev = mail_tool_remove_xevolution_headers (message);
 	mail_tool_destroy_xevolution (xev);
 	
+	/* remove Bcc headers */
+	while (camel_medium_get_header (CAMEL_MEDIUM (message), "Bcc"))
+		camel_medium_remove_header (CAMEL_MEDIUM (message), "Bcc");
+	
 	part = camel_mime_part_new ();
 	camel_mime_part_set_disposition (part, "inline");
 	camel_mime_part_set_description (part, desc);
@@ -338,11 +349,15 @@ mail_tool_quote_message (CamelMimeMessage *message, const char *fmt, ...)
 	gboolean want_plain;
 	gchar *text;
 	
-	want_plain = !mail_config_get_send_html ();
 	contents = camel_medium_get_content_object (CAMEL_MEDIUM (message));
 	/* We pass "want_plain" for "cite", since if it's HTML, we'll
 	 * do the citing ourself below.
 	 */
+	/* FIXME the citing logic has changed and we basically never want_plain
+	 * to be true now, but I don't want to remove all that logic until I
+	 * am sure --Larry
+	 */
+	want_plain = FALSE;
 	text = mail_get_message_body (contents, want_plain, want_plain);
 	
 	/* Set the quoted reply text. */
@@ -363,7 +378,7 @@ mail_tool_quote_message (CamelMimeMessage *message, const char *fmt, ...)
 					    "<!--+GtkHTML:<DATA class=\"ClueFlow\" clear=\"orig\">-->",
 					    credits ? credits : "",
 					    mail_config_get_citation_color (),
-					    want_plain ? "" : "<blockquote><i>",
+					    want_plain ? "" : "<blockquote type=cite><i>",
 					    text,
 					    want_plain ? "" : "</i></blockquote>");
 		g_free (text);
@@ -385,7 +400,7 @@ mail_tool_quote_message (CamelMimeMessage *message, const char *fmt, ...)
 gchar *
 mail_tool_forward_message (CamelMimeMessage *message, gboolean quoted)
 {
-	gchar *title, *body, *ret;
+	char *title, *body, *ret;
 	
 	body = mail_get_message_body (CAMEL_DATA_WRAPPER (message),
 				      !mail_config_get_send_html (),
@@ -438,3 +453,26 @@ mail_tools_x_evolution_message_parse (char *in, unsigned int inlen, GPtrArray **
 	
 	return folder;
 }
+
+
+char *
+mail_tools_folder_to_url (CamelFolder *folder)
+{
+	char *service_url, *url;
+	const char *full_name;
+	CamelService *service;
+	
+	g_return_val_if_fail (CAMEL_IS_FOLDER (folder), NULL);
+	
+	full_name = folder->full_name;
+	while (*full_name == '/')
+		full_name++;
+	
+	service = (CamelService *) folder->parent_store;
+	service_url = camel_url_to_string (service->url, CAMEL_URL_HIDE_ALL);
+	url = g_strdup_printf ("%s%s%s", service_url, service_url[strlen (service_url)-1] != '/' ? "/" : "",
+			       full_name);
+	g_free (service_url);
+	
+	return url;
+}
