@@ -181,11 +181,20 @@ folder_list_from_vector (const char *vector)
 
 	for (tokens = t; *tokens; tokens += 2) {
 		ESummaryPrefsFolder *folder;
+		const char *evolution_uri;
+		const char *physical_uri;
+
+		evolution_uri = *tokens;
+		if (evolution_uri == NULL || strncmp (evolution_uri, "evolution:", 10) != 0)
+			break;
+
+		physical_uri = *(tokens + 1);
+		if (physical_uri == NULL)
+			break;
 
 		folder = g_new (ESummaryPrefsFolder, 1);
-		g_print ("%s - %s\n", *tokens, *(tokens + 1));
-		folder->evolution_uri = g_strdup (*tokens);
-		folder->physical_uri = g_strdup (*(tokens + 1));
+		folder->evolution_uri = g_strdup (evolution_uri);
+		folder->physical_uri = g_strdup (physical_uri);
 
 		flist = g_list_prepend (flist, folder);
 	}
@@ -222,7 +231,6 @@ vector_from_folder_list (GList *flist)
 
 	vector = string->str;
 	g_string_free (string, FALSE);
-	g_print ("vector: %s\n", vector);
 
 	return vector;
 }
@@ -573,8 +581,6 @@ static struct _RDFInfo rdfs[] = {
 	{"http://barrapunto.com/gnome.rdf", "Barrapunto GNOME", FALSE,},
 	{"http://www.bsdtoday.com/backend/bt.rdf", "BSD Today", FALSE},
 	{"http://beyond2000.com/b2k.rdf", "Beyond 2000", FALSE},
-  	{"http://www.cnn.com/cnn.rss", "CNN", FALSE},
-        {"http://www.debianplanet.org/debianplanet/backend.php", "Debian Planet", FALSE},
 	{"http://www.dictionary.com/wordoftheday/wotd.rss", N_("Dictionary.com Word of the Day"), FALSE},
 	{"http://www.dvdreview.com/rss/newschannel.rss", "DVD Review", FALSE},
 	{"http://freshmeat.net/backend/fm.rdf", "Freshmeat", FALSE},
@@ -598,7 +604,7 @@ static struct _RDFInfo rdfs[] = {
 	{"http://slashdot.org/slashdot.rdf", "Slashdot", FALSE},
 	{"http://www.theregister.co.uk/tonys/slashdot.rdf", "The Register", FALSE},
 	{"http://www.webreference.com/webreference.rdf", "Web Reference", FALSE},
-	{"http://redcarpet.ximian.com/red-carpet.rdf", "Ximian Red Carpet New", FALSE},
+	{"http://redcarpet.ximian.com/red-carpet.rdf", "Ximian Red Carpet News", FALSE},
 	{NULL, NULL, FALSE}
 };
 
