@@ -4,9 +4,8 @@
  * Copyright (C) 2000  Ximian, Inc.
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * modify it under the terms of version 2 of the GNU General Public
+ * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -65,14 +64,12 @@ void        e_shell_folder_selection_dialog_construct          (EShellFolderSele
 								const char                  *title,
 								const char                  *caption,
 								const char                  *default_uri,
-								const char                  *allowed_types[],
-								const char                  *default_type);
+								const char                  *allowed_types[]);
 GtkWidget  *e_shell_folder_selection_dialog_new                (EShell                      *shell,
 								const char                  *title,
 								const char                  *caption,
 								const char                  *default_uri,
-								const char                  *allowed_types[],
-								const char                  *default_type);
+								const char                  *allowed_types[]);
 
 void      e_shell_folder_selection_dialog_set_allow_creation  (EShellFolderSelectionDialog *folder_selection_dialog,
 							       gboolean                     allow_creation);
