@@ -49,6 +49,23 @@ struct _EShellSettingsDialogPrivate {
 	GHashTable *types;
 };
 
+
+/* FIXME ugly hack to work around that sizing of invisible widgets is broken
+   with Bonobo.  */
+
+static void
+set_dialog_size (EShellSettingsDialog *dialog)
+{
+	GdkFont *font;
+	int width, height;
+
+	font = GTK_WIDGET (dialog)->style->font;
+	width = gdk_string_width (font, "M") * 66;
+	height = (font->ascent + font->descent) * 35;
+
+	gtk_widget_set_usize (GTK_WIDGET (dialog), width, height);
+}
+
 
 /* Page handling.  */
 
@@ -76,8 +93,8 @@ page_new (const char *title,
 		gdk_pixbuf_ref (icon);
 
 	page = g_new (Page, 1);
-	page->title       = g_strdup (title);
-	page->description = g_strdup (description);
+	page->title       = g_strdup (_(title));
+	page->description = g_strdup (_(description));
 	page->icon        = icon;
 	page->type        = type;
 	page->priority    = priority;
@@ -235,6 +252,21 @@ load_pages (EShellSettingsDialog *dialog)
 	CORBA_exception_free (&ev);
 }
 
+
+/* GtkWidget methods.  */
+
+static void
+impl_realize (GtkWidget *widget)
+{
+	EShellSettingsDialog *dialog;
+
+	dialog = E_SHELL_SETTINGS_DIALOG (widget);
+
+	set_dialog_size (dialog);
+
+	(* GTK_WIDGET_CLASS (parent_class)->realize) (widget);
+}
+
 
 /* GtkObject methods.  */
 
@@ -268,10 +300,14 @@ static void
 class_init (EShellSettingsDialog *class)
 {
 	GtkObjectClass *object_class;
+	GtkWidgetClass *widget_class;
 
 	object_class = GTK_OBJECT_CLASS (class);
 	object_class->destroy = impl_destroy;
 
+	widget_class = GTK_WIDGET_CLASS (class);
+	widget_class->realize = impl_realize;
+
 	parent_class = gtk_type_class (PARENT_TYPE);
 }
 
@@ -288,7 +324,6 @@ init (EShellSettingsDialog *dialog)
 	load_pages (dialog);
 
 	gtk_window_set_title (GTK_WINDOW (dialog), _("Evolution Settings"));
-	gtk_window_set_default_size (GTK_WINDOW (dialog), 600, 450);
 }
 
 
@@ -306,7 +341,7 @@ void
 e_shell_settings_dialog_show_type (EShellSettingsDialog *dialog, const char *type)
 {
 	EShellSettingsDialogPrivate *priv;
-	gpointer value;
+	gpointer key, value;
 	int page;
 	
 	g_return_if_fail (dialog != NULL);
@@ -315,7 +350,17 @@ e_shell_settings_dialog_show_type (EShellSettingsDialog *dialog, const char *typ
 
 	priv = dialog->priv;
 	
-	value = g_hash_table_lookup (priv->types, type);
+	if (!g_hash_table_lookup_extended (priv->types, type, &key, &value)) {
+		char *slash, *supertype;
+
+		slash = strchr (type, '/');
+		if (slash) {
+			supertype = g_strndup (type, slash - type);
+			value = g_hash_table_lookup (priv->types, type);
+			g_free (supertype);
+		} else
+			value = NULL;
+	}
 	page = GPOINTER_TO_INT (value);
 	
 	e_multi_config_dialog_show_page (E_MULTI_CONFIG_DIALOG (dialog), page);
