@@ -78,6 +78,30 @@ get_path_for_folder_op (EShellView *shell_view)
 	return e_shell_view_get_current_path (shell_view);
 }
 
+static void
+launch_pilot_settings (const char *extra_arg)
+{
+        char *args[] = {
+                "gpilotd-control-applet",
+		(char *) extra_arg,
+		NULL
+        };
+        int pid;
+
+        args[0] = gnome_is_program_in_path ("gpilotd-control-applet");
+        if (!args[0]) {
+		e_notice (NULL, GNOME_MESSAGE_BOX_ERROR,
+			  _("The GNOME Pilot tools do not appear to be installed on this system."));
+		return;
+        }
+
+        pid = gnome_execute_async (NULL, extra_arg ? 2 : 1, args);
+        g_free (args[0]);
+
+        if (pid == -1)
+                e_notice (NULL, GNOME_MESSAGE_BOX_ERROR, _("Error executing %s."), args[0]);
+}
+
 
 /* EShellView callbacks.  */
 
@@ -130,17 +154,11 @@ command_close (BonoboUIComponent *uih,
 	       const char *path)
 {
 	EShellView *shell_view;
-	GdkEvent delete_event;
 
 	shell_view = E_SHELL_VIEW (data);
 
-	/* Send a delete_event to the window.  This way we make sure that the
-	   behaviors for delete_event and the menu item are the same.  */
-
-	delete_event.any.type = GDK_DELETE;
-	delete_event.any.window = GTK_WIDGET (shell_view)->window;
-	delete_event.any.send_event = 0;
-	gtk_widget_event (GTK_WIDGET (shell_view), &delete_event);
+	if (e_shell_request_close_view (e_shell_view_get_shell (shell_view), shell_view))
+		gtk_widget_destroy (GTK_WIDGET (shell_view));
 }
 
 static void
@@ -411,7 +429,7 @@ command_add_folder_to_shortcut_bar (BonoboUIComponent *uih,
 }
 
 
-/* Opening other users' folders.  */
+/* Opening and removing other users' folders.  */
 
 static void
 command_open_other_users_folder (BonoboUIComponent *uih,
@@ -427,6 +445,18 @@ command_open_other_users_folder (BonoboUIComponent *uih,
 	e_shell_show_shared_folder_picker_dialog (shell, shell_view);
 }
 
+static void
+command_remove_other_users_folder (BonoboUIComponent *uih,
+				   void *data,
+				   const char *path)
+{
+	EShellView *shell_view;
+
+	shell_view = E_SHELL_VIEW (data);
+	e_shell_command_remove_shared_folder (e_shell_view_get_shell (shell_view), shell_view,
+					      get_path_for_folder_op (shell_view));
+}
+
 
 /* Going to a folder.  */
 
@@ -627,24 +657,15 @@ command_pilot_settings (BonoboUIComponent *uih,
 			void *data,
 			const char *path)
 {
-        char *args[] = {
-                "gpilotd-control-applet",
-                NULL
-        };
-        int pid;
+	launch_pilot_settings (NULL);
+}
 
-        args[0] = gnome_is_program_in_path ("gpilotd-control-applet");
-        if (!args[0]) {
-		e_notice (NULL, GNOME_MESSAGE_BOX_ERROR,
-			  _("The GNOME Pilot tools do not appear to be installed on this system."));
-		return;
-        }
-
-        pid = gnome_execute_async (NULL, 4, args);
-        g_free (args[0]);
-
-        if (pid == -1)
-                e_notice (NULL, GNOME_MESSAGE_BOX_ERROR, _("Error executing %s."), args[0]);
+static void
+command_pilot_conduit_settings (BonoboUIComponent *uih,
+				void *data,
+				const char *path)
+{
+	launch_pilot_settings ("--cap-id=1");
 }
 
 
@@ -658,6 +679,7 @@ static BonoboUIVerb new_verbs [] = {
 static BonoboUIVerb file_verbs [] = {
 	BONOBO_UI_VERB ("FileImporter", (BonoboUIVerbFn) show_import_wizard),
 	BONOBO_UI_VERB ("FileOpenOtherUsersFolder", command_open_other_users_folder),
+	BONOBO_UI_VERB ("FileRemoveOtherUsersFolder", command_remove_other_users_folder),
 	BONOBO_UI_VERB ("FileGoToFolder", command_goto_folder),
 	BONOBO_UI_VERB ("FileCreateFolder", command_create_folder),
 	BONOBO_UI_VERB ("FileClose", command_close),
@@ -692,6 +714,7 @@ static BonoboUIVerb actions_verbs[] = {
 static BonoboUIVerb tools_verbs[] = {
 	BONOBO_UI_VERB ("Settings", command_settings),
 
+	BONOBO_UI_VERB ("PilotConduitSettings", command_pilot_conduit_settings),
 	BONOBO_UI_VERB ("PilotSettings", command_pilot_settings),
 
 	BONOBO_UI_VERB_END
