@@ -27,6 +27,27 @@
 #include <config.h>
 #endif
 
+#include "e-shell-view.h"
+
+#include "evolution-shell-view.h"
+
+#include "e-gray-bar.h"
+#include "e-history.h"
+#include "e-icon-factory.h"
+#include "e-shell-constants.h"
+#include "e-shell-folder-title-bar.h"
+#include "e-shell-utils.h"
+#include "e-shell-view-menu.h"
+#include "e-shell.h"
+#include "e-shortcuts-view.h"
+#include "e-storage-set-view.h"
+#include "e-title-bar.h"
+
+#include "e-util/e-gtk-utils.h"
+
+#include "widgets/misc/e-clipped-label.h"
+#include "widgets/misc/e-bonobo-widget.h"
+
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <fcntl.h>
@@ -48,25 +69,6 @@
 #include <gal/widgets/e-unicode.h>
 #include <gal/widgets/e-scroll-frame.h>
 
-#include "widgets/misc/e-clipped-label.h"
-#include "widgets/misc/e-bonobo-widget.h"
-
-#include "e-util/e-gtk-utils.h"
-
-#include "evolution-shell-view.h"
-
-#include "e-gray-bar.h"
-#include "e-shell-constants.h"
-#include "e-shell-folder-title-bar.h"
-#include "e-shell-utils.h"
-#include "e-shell.h"
-#include "e-shortcuts-view.h"
-#include "e-storage-set-view.h"
-#include "e-title-bar.h"
-
-#include "e-shell-view.h"
-#include "e-shell-view-menu.h"
-
 
 static BonoboWindowClass *parent_class = NULL;
 
@@ -89,6 +91,9 @@ struct _EShellViewPrivate {
 	BonoboUIComponent *ui_component;
 	BonoboUIContainer *ui_container;
 
+	/* History of visited (evolution:) URIs. */
+	EHistory *history;
+
 	/* Currently displayed URI.  */
 	char *uri;
 
@@ -147,6 +152,7 @@ struct _EShellViewPrivate {
 enum {
 	SHORTCUT_BAR_VISIBILITY_CHANGED,
 	FOLDER_BAR_VISIBILITY_CHANGED,
+	VIEW_CHANGED,
 	LAST_SIGNAL
 };
 
@@ -164,7 +170,7 @@ static guint signals[LAST_SIGNAL] = { 0 };
 
 /* URI to display when the currently displayed folder is removed from the
    storage.  */
-#define FALLBACK_URI "evolution:/local/Inbox"
+#define FALLBACK_URI E_SUMMARY_URI
 
 
 /* The icons for the offline/online status.  */
@@ -183,6 +189,8 @@ static const char *get_storage_set_path_from_uri  (const char *uri);
 
 /* Boo.  */
 static void new_folder_cb (EStorageSet *storage_set, const char *path, void *data);
+static gboolean display_uri (EShellView *shell_view, const char *uri,
+			     gboolean add_to_history, gboolean queue);
 
 
 /* View handling.  */
@@ -287,31 +295,78 @@ setup_verb_sensitivity_for_folder (EShellView *shell_view,
 {
 	EShellViewPrivate *priv;
 	BonoboUIComponent *ui_component;
+	EFolder *folder;
 	const char *prop;
 
 	priv = shell_view->priv;
 
+	ui_component = e_shell_view_get_bonobo_ui_component (shell_view);
+
+	if (path == NULL)
+		folder = NULL;
+	else
+		folder = e_storage_set_get_folder (e_shell_get_storage_set (priv->shell), path);
+
 	/* Adjust sensitivity for menu options depending on whether the folder
            selected is a stock folder.  */
 
-	if (path == NULL) {
+	if (folder != NULL && ! e_folder_get_is_stock (folder))
+		prop = "1";
+	else
 		prop = "0";
-	} else {
-		EFolder *folder;
-
-		folder = e_storage_set_get_folder (e_shell_get_storage_set (priv->shell), path);
-		if (folder != NULL && ! e_folder_get_is_stock (folder))
-			prop = "1";
-		else
-			prop = "0";
-	}
-
-	ui_component = e_shell_view_get_bonobo_ui_component (shell_view);
-
 	bonobo_ui_component_set_prop (ui_component, "/commands/MoveFolder", "sensitive", prop, NULL);
 	bonobo_ui_component_set_prop (ui_component, "/commands/CopyFolder", "sensitive", prop, NULL);
 	bonobo_ui_component_set_prop (ui_component, "/commands/DeleteFolder", "sensitive", prop, NULL);
 	bonobo_ui_component_set_prop (ui_component, "/commands/RenameFolder", "sensitive", prop, NULL);
+
+	/* Adjust sensitivity for menu options depending on whether the user
+	   right-clicked a folder whose contents can be viewed.  */
+
+	if (folder != NULL
+	    && e_folder_type_registry_get_handler_for_type (e_shell_get_folder_type_registry (e_shell_view_get_shell (shell_view)),
+							    e_folder_get_type_string (folder)) != NULL)
+		prop = "1";
+	else
+		prop = "0";
+	bonobo_ui_component_set_prop (ui_component, "/commands/ActivateView", "sensitive", prop, NULL);
+	bonobo_ui_component_set_prop (ui_component, "/commands/OpenFolderInNewWindow", "sensitive", prop, NULL);
+	bonobo_ui_component_set_prop (ui_component, "/commands/AddFolderToShortcutBar", "sensitive", prop, NULL);
+}
+
+
+static void
+update_navigation_buttons (EShellView *shell_view)
+{
+	EShellViewPrivate *priv;
+
+	priv = shell_view->priv;
+
+	e_shell_folder_title_bar_update_navigation_buttons (E_SHELL_FOLDER_TITLE_BAR (priv->folder_title_bar),
+							    e_history_has_prev (priv->history),
+							    e_history_has_next (priv->history));
+}
+
+static int
+history_uri_matching_func (const void *a,
+			   const void *b)
+{
+	const char *s1, *s2;
+
+	s1 = (const char *) a;
+	s2 = (const char *) b;
+
+	return strcmp (s1, s2);
+}
+
+static void
+remove_uri_from_history (EShellView *shell_view,
+			 const char *uri)
+{
+	EShellViewPrivate *priv;
+
+	priv = shell_view->priv;
+	
+	e_history_remove_matching (priv->history, uri, history_uri_matching_func);
 }
 
 
@@ -369,22 +424,22 @@ handle_current_folder_removed (EShellView *shell_view)
 
 	current_path = priv->uri + E_SHELL_URI_PREFIX_LEN;
 
-	g_assert (*current_path == G_DIR_SEPARATOR);
+	g_assert (*current_path == E_PATH_SEPARATOR);
 
 	new_path = NULL;
 
 	/* If we have a parent folder (not a parent storage), try to display
 	   that one.  */
 
-	p = strrchr (current_path + 1, G_DIR_SEPARATOR);
-	if (p != NULL && p[1] != '\0' && strchr (current_path + 1, G_DIR_SEPARATOR) != p) {
+	p = strrchr (current_path + 1, E_PATH_SEPARATOR);
+	if (p != NULL && p[1] != '\0' && strchr (current_path + 1, E_PATH_SEPARATOR) != p) {
 		new_path = g_strndup (current_path, p - current_path);
 	} else {
 		/* We don't have a parent folder, so try to see if there is an
 		   Inbox folder in the same storage.  */
 
 		/* Extract the storage name.  */
-		p = strchr (current_path + 1, G_DIR_SEPARATOR);
+		p = strchr (current_path + 1, E_PATH_SEPARATOR);
 		if (p == NULL) {
 			/* The URL points itself to a storage, so just redirect
 			   to the default case.  */
@@ -400,7 +455,7 @@ handle_current_folder_removed (EShellView *shell_view)
 
 				/* No Inbox in this storage -- fallback to the storage.  */
 				storage_uri = g_strconcat (E_SHELL_URI_PREFIX, storage_name, NULL);
-				e_shell_view_display_uri (shell_view, storage_uri);
+				e_shell_view_display_uri (shell_view, storage_uri, TRUE);
 
 				g_free (storage_uri);
 				g_free (storage_name);
@@ -412,7 +467,7 @@ handle_current_folder_removed (EShellView *shell_view)
 	}
 
 	if (new_path == NULL) {
-		e_shell_view_display_uri (shell_view, FALLBACK_URI);
+		e_shell_view_display_uri (shell_view, FALLBACK_URI, TRUE);
 	} else {
 		EFolder *folder;
 
@@ -421,12 +476,12 @@ handle_current_folder_removed (EShellView *shell_view)
 
 		folder = e_storage_set_get_folder (e_shell_get_storage_set (priv->shell), new_path);
 		if (folder == NULL) {
-			e_shell_view_display_uri (shell_view, FALLBACK_URI);
+			e_shell_view_display_uri (shell_view, FALLBACK_URI, TRUE);
 		} else {
 			char *new_uri;
 
 			new_uri = g_strconcat (E_SHELL_URI_PREFIX, new_path, NULL);
-			e_shell_view_display_uri (shell_view, new_uri);
+			e_shell_view_display_uri (shell_view, new_uri, TRUE);
 			g_free (new_uri);
 		}
 
@@ -454,7 +509,16 @@ storage_set_removed_folder_callback (EStorageSet *storage_set,
 	priv = shell_view->priv;
 
 	uri = g_strconcat (E_SHELL_URI_PREFIX, path, NULL);
+
+	remove_uri_from_history (shell_view, uri);
+	update_navigation_buttons (shell_view);
+
+	/* (Note that at this point the current URI in the history might have
+	   been changed and not match the current view.  But we catch this case
+	   when checking if this was the current view, below.)  */
+
 	view = g_hash_table_lookup (priv->uri_to_view, uri);
+
 	g_free (uri);
 
 	if (view == NULL)
@@ -680,9 +744,9 @@ set_folder_timeout (gpointer data)
 	shell_view = E_SHELL_VIEW (data);
 	priv = shell_view->priv;
 
-	/* set to 0 so we don't remove it in _display_uri() */
+	/* Set to 0 so we don't remove it in _display_uri().  */
 	priv->set_folder_timeout = 0;
-	e_shell_view_display_uri (shell_view, priv->set_folder_uri);
+	e_shell_view_display_uri (shell_view, priv->set_folder_uri, TRUE);
 
 	return FALSE;
 }
@@ -729,7 +793,7 @@ switch_on_folder_tree_click (EShellView *shell_view,
 	cleanup_delayed_selection (shell_view);
 
 	if (priv->folder_bar_popup != NULL) {
-		e_shell_view_display_uri (shell_view, uri);
+		e_shell_view_display_uri (shell_view, uri, TRUE);
 		g_free (uri);
 
 		gtk_object_ref (GTK_OBJECT (shell_view));
@@ -745,8 +809,8 @@ switch_on_folder_tree_click (EShellView *shell_view,
 
 /* Callbacks.  */
 
-/* Callback when a new folder is added.  removed when we clear the
-   delayed_selection */
+/* Callback when a new folder is added.  Removed when we clear the
+   delayed_selection.  */
 static void
 new_folder_cb (EStorageSet *storage_set,
 	       const char *path,
@@ -767,7 +831,7 @@ new_folder_cb (EStorageSet *storage_set,
 
 			uri = g_strdup (priv->delayed_selection);
 			cleanup_delayed_selection (shell_view);
-			e_shell_view_display_uri (shell_view, uri);
+			e_shell_view_display_uri (shell_view, uri, FALSE);
 			g_free (uri);
 		}
 	}
@@ -792,7 +856,7 @@ activate_shortcut_cb (EShortcutsView *shortcut_view,
 		e_shell_view_show_shortcut_bar (new_view, FALSE);
 		e_shell_view_show_folder_bar (new_view, FALSE);
 	} else {
-		e_shell_view_display_uri (shell_view, uri);
+		e_shell_view_display_uri (shell_view, uri, TRUE);
 	}
 }
 
@@ -822,23 +886,6 @@ folder_selected_cb (EStorageSetView *storage_set_view,
 	switch_on_folder_tree_click (shell_view, path);
 }
 
-/* Callback called when a storage in the tree view is clicked.  */
-static void
-storage_selected_cb (EStorageSetView *storage_set_view,
-		     const char *name,
-		     void *data)
-{
-	EShellView *shell_view;
-	char *path;
-
-	shell_view = E_SHELL_VIEW (data);
-
-	path = g_strconcat (G_DIR_SEPARATOR_S, name, NULL);
-	switch_on_folder_tree_click (shell_view, path);
-
-	g_free (path);
-}
-
 /* Callbacks for the folder context menu in the folder bar.  */
 
 static void
@@ -920,6 +967,47 @@ offline_toggle_clicked_cb (GtkButton *button,
 	}
 }
 
+
+/* Navigation button callbacks.  */
+
+static void
+back_clicked_callback (EShellFolderTitleBar *title_bar,
+		       void *data)
+{
+	EShellView *shell_view;
+	EShellViewPrivate *priv;
+	const char *new_uri;
+
+	shell_view = E_SHELL_VIEW (data);
+	priv = shell_view->priv;
+
+	if (! e_history_has_prev (priv->history))
+		return;
+
+	new_uri = (const char *) e_history_prev (priv->history);
+
+	display_uri (shell_view, new_uri, FALSE, TRUE);
+}
+
+static void
+forward_clicked_callback (EShellFolderTitleBar *title_bar,
+			  void *data)
+{
+	EShellView *shell_view;
+	EShellViewPrivate *priv;
+	const char *new_uri;
+
+	shell_view = E_SHELL_VIEW (data);
+	priv = shell_view->priv;
+
+	if (! e_history_has_next (priv->history))
+		return;
+
+	new_uri = (const char *) e_history_next (priv->history);
+
+	display_uri (shell_view, new_uri, FALSE, TRUE);
+}
+
 
 /* Widget setup.  */
 
@@ -933,12 +1021,10 @@ setup_storage_set_subwindow (EShellView *shell_view)
 
 	priv = shell_view->priv;
 
-	storage_set_view = e_storage_set_view_new (e_shell_get_storage_set (priv->shell),
-						   priv->ui_container);
+	storage_set_view = e_storage_set_create_new_view (e_shell_get_storage_set (priv->shell),
+							  priv->ui_container);
 	gtk_signal_connect (GTK_OBJECT (storage_set_view), "folder_selected",
 			    GTK_SIGNAL_FUNC (folder_selected_cb), shell_view);
-	gtk_signal_connect (GTK_OBJECT (storage_set_view), "storage_selected",
-			    GTK_SIGNAL_FUNC (storage_selected_cb), shell_view);
 	gtk_signal_connect (GTK_OBJECT (storage_set_view), "folder_context_menu_popping_up",
 			    GTK_SIGNAL_FUNC (folder_context_menu_popping_up_cb), shell_view);
 	gtk_signal_connect (GTK_OBJECT (storage_set_view), "folder_context_menu_popped_down",
@@ -1143,6 +1229,10 @@ setup_widgets (EShellView *shell_view)
 	priv->folder_title_bar = e_shell_folder_title_bar_new ();
 	gtk_signal_connect (GTK_OBJECT (priv->folder_title_bar), "title_toggled",
 			    GTK_SIGNAL_FUNC (title_bar_toggled_cb), shell_view);
+	gtk_signal_connect (GTK_OBJECT (priv->folder_title_bar), "back_clicked",
+			    GTK_SIGNAL_FUNC (back_clicked_callback), shell_view);
+	gtk_signal_connect (GTK_OBJECT (priv->folder_title_bar), "forward_clicked",
+			    GTK_SIGNAL_FUNC (forward_clicked_callback), shell_view);
 
 	priv->view_hpaned = e_hpaned_new ();
 	e_paned_pack1 (E_PANED (priv->view_hpaned), priv->storage_set_view_box, FALSE, FALSE);
@@ -1228,6 +1318,9 @@ destroy (GtkObject *object)
 
 	gtk_object_unref (GTK_OBJECT (priv->tooltips));
 
+	if (priv->history != NULL)
+		gtk_object_unref (GTK_OBJECT (priv->history));
+
 	if (priv->shell != NULL)
 		bonobo_object_unref (BONOBO_OBJECT (priv->shell));
 
@@ -1298,6 +1391,18 @@ class_init (EShellViewClass *klass)
 				  GTK_TYPE_NONE, 1,
 				  GTK_TYPE_INT);
 
+	signals[VIEW_CHANGED]
+		= gtk_signal_new ("view_changed",
+				  GTK_RUN_FIRST,
+				  object_class->type,
+				  GTK_SIGNAL_OFFSET (EShellViewClass, view_changed),
+				  e_marshal_NONE__POINTER_POINTER_POINTER_POINTER,
+				  GTK_TYPE_NONE, 4,
+				  GTK_TYPE_STRING,
+				  GTK_TYPE_STRING,
+				  GTK_TYPE_STRING,
+				  GTK_TYPE_STRING);
+
 	gtk_object_class_add_signals (object_class, signals, LAST_SIGNAL);
 
 	load_images ();
@@ -1313,6 +1418,7 @@ init (EShellView *shell_view)
 	priv->shell                   = NULL;
 	priv->corba_interface         = NULL;
 	priv->ui_component            = NULL;
+	priv->history                 = e_history_new ((EHistoryItemFreeFunc) g_free);
 	priv->uri                     = NULL;
 	priv->delayed_selection       = NULL;
 
@@ -1385,7 +1491,7 @@ corba_interface_change_current_view_cb (EvolutionShellView *shell_view,
 
 	g_return_if_fail (view != NULL);
 
-	e_shell_view_display_uri (view, uri);
+	e_shell_view_display_uri (view, uri, TRUE);
 }
 
 static void
@@ -1420,6 +1526,22 @@ corba_interface_set_folder_bar_label (EvolutionShellView *evolution_shell_view,
 						       text);
 }
 
+static void
+corba_interface_show_settings (EvolutionShellView *evolution_shell_view,
+			       void *data)
+{
+	EShellView *shell_view;
+	EShellViewPrivate *priv;
+
+	g_return_if_fail (data != NULL);
+	g_return_if_fail (E_IS_SHELL_VIEW (data));
+
+	shell_view = E_SHELL_VIEW (data);
+	priv = shell_view->priv;
+
+	e_shell_view_show_settings (shell_view);
+}
+
 static void
 unmerge_on_error (BonoboObject *object,
 		  CORBA_Object  cobject,
@@ -1465,6 +1587,15 @@ shell_line_status_changed_cb (EShell *shell,
 
 	shell_view = E_SHELL_VIEW (data);
 	update_offline_toggle_status (shell_view);
+
+	if (new_status == E_SHELL_LINE_STATUS_OFFLINE)
+		bonobo_ui_component_set_prop (shell_view->priv->ui_component,
+					      "/commands/SendReceive",
+					      "sensitive", "0", NULL);
+	else
+		bonobo_ui_component_set_prop (shell_view->priv->ui_component,
+					      "/commands/SendReceive",
+					      "sensitive", "1", NULL);
 }
 
 static int
@@ -1538,8 +1669,8 @@ e_shell_view_construct (EShellView *shell_view,
 					GTK_SIGNAL_FUNC (storage_set_removed_folder_callback), shell_view,
 					GTK_OBJECT (shell_view));
 
-	e_shell_user_creatable_items_handler_setup_menus (e_shell_get_user_creatable_items_handler (priv->shell),
-							  shell_view);
+	e_shell_user_creatable_items_handler_attach_menus (e_shell_get_user_creatable_items_handler (priv->shell),
+							   shell_view);
 
 	return view;
 }
@@ -1634,24 +1765,37 @@ update_window_icon (EShellView *shell_view,
 static void
 update_folder_title_bar (EShellView *shell_view,
 			 const char *title,
-			 const char *type)
+			 EFolder *folder)
 {
 	EShellViewPrivate *priv;
-	EFolderTypeRegistry *folder_type_registry;
 	GdkPixbuf *folder_icon;
 
 	priv = shell_view->priv;
 
-	if (type == NULL) {
-		title = NULL;
+	if (folder == NULL) {
 		folder_icon = NULL;
 	} else {
-		folder_type_registry = e_shell_get_folder_type_registry (priv->shell);
-		folder_icon = e_folder_type_registry_get_icon_for_type (folder_type_registry, type, TRUE);
+		const char *icon_name;
+
+		icon_name = e_folder_get_custom_icon_name (folder);
+		if (icon_name != NULL) {
+			folder_icon = e_icon_factory_get_icon (icon_name, TRUE);
+		} else {
+			EFolderTypeRegistry *folder_type_registry;
+
+			folder_type_registry = e_shell_get_folder_type_registry (priv->shell);
+			folder_icon = e_folder_type_registry_get_icon_for_type (folder_type_registry,
+										e_folder_get_type_string (folder),
+										TRUE);
+			gdk_pixbuf_ref (folder_icon);
+		}
 	}
 
+	e_shell_folder_title_bar_set_icon (E_SHELL_FOLDER_TITLE_BAR (priv->folder_title_bar),
+					   folder_icon);
+
 	if (folder_icon != NULL)
-		e_shell_folder_title_bar_set_icon (E_SHELL_FOLDER_TITLE_BAR (priv->folder_title_bar), folder_icon);
+		gdk_pixbuf_unref (folder_icon);
 
 	if (title != NULL) {
 		char *s;
@@ -1677,39 +1821,26 @@ update_for_current_uri (EShellView *shell_view)
 
 	priv = shell_view->priv;
 
-	/* if we update when there is a timeout set, the selection
-	 * will jump around against the user's wishes.  so we just
-	 * return.
-	 */     
+	/* If we update when there is a timeout set, the selection will jump
+	  around against the user's wishes.  So we just return.  */     
 	if (priv->set_folder_timeout != 0)
 		return;
 
 	path = get_storage_set_path_from_uri (priv->uri);
 
+	folder = NULL;
 	folder_name = NULL;
 	type = NULL;
 	unread_count = 0;
 
-	if (path == NULL) {
-		folder = NULL;
-	} else {
+	if (path != NULL) {
 		folder = e_storage_set_get_folder (e_shell_get_storage_set (priv->shell), path);
 
 		if (folder != NULL) {
 			folder_name = e_folder_get_name (folder);
 			type = e_folder_get_type_string (folder);
 			unread_count = e_folder_get_unread_count (folder);
-		} else if (path != NULL) {
-			EStorage *storage;
-
-			storage = e_storage_set_get_storage (e_shell_get_storage_set (priv->shell), path + 1);
-			unread_count = 0;
-
-			if (storage != NULL) {
-				folder_name = e_storage_get_display_name (storage);
-				type = e_storage_get_toplevel_node_type (storage);
-			}
-		} 
+		}
 	}
 
 	if (unread_count > 0)
@@ -1727,7 +1858,7 @@ update_for_current_uri (EShellView *shell_view)
 	gtk_window_title = e_utf8_to_gtk_string (GTK_WIDGET (shell_view), utf8_window_title);
 	gtk_window_set_title (GTK_WINDOW (shell_view), gtk_window_title);
 
-	update_folder_title_bar (shell_view, title, type);
+	update_folder_title_bar (shell_view, title, folder);
 	update_window_icon (shell_view, type);
 
 	g_free (gtk_window_title);
@@ -1810,7 +1941,6 @@ set_current_notebook_page (EShellView *shell_view,
 
 	if (current_page != -1 && current_page != 0) {
 		current = gtk_notebook_get_nth_page (notebook, current_page);
-
 		old_control_frame = bonobo_widget_get_control_frame (BONOBO_WIDGET (current));
 		bonobo_control_frame_set_autoactivate (old_control_frame, FALSE);
 	}
@@ -1860,6 +1990,9 @@ setup_corba_interface (EShellView *shell_view,
 	gtk_signal_connect_while_alive (GTK_OBJECT (corba_interface), "set_folder_bar_label",
 					GTK_SIGNAL_FUNC (corba_interface_set_folder_bar_label),
 					shell_view, GTK_OBJECT (shell_view));
+	gtk_signal_connect_while_alive (GTK_OBJECT (corba_interface), "show_settings",
+					GTK_SIGNAL_FUNC (corba_interface_show_settings),
+					shell_view, GTK_OBJECT (shell_view));
 
 	bonobo_object_add_interface (BONOBO_OBJECT (control_frame),
 				     BONOBO_OBJECT (corba_interface));
@@ -1905,17 +2038,10 @@ socket_destroy_cb (GtkWidget *socket_widget, gpointer data)
 	path = get_storage_set_path_from_uri (uri);
 	folder = e_storage_set_get_folder (e_shell_get_storage_set (priv->shell), path);
 
-	if (folder != NULL) {
+	if (folder != NULL)
 		folder_type = e_folder_get_type_string (folder);
-	} else {
-		EStorage *storage;
-
-		storage = e_storage_set_get_storage (e_shell_get_storage_set (priv->shell), path + 1);
-		if (storage == NULL)
-			folder_type = NULL;
-		else
-			folder_type = e_storage_get_toplevel_node_type (storage);
-	}
+	else
+		folder_type = NULL;
 
 	/* See if we were actively viewing the uri for the socket that's being closed */
 	current_uri = e_shell_view_get_current_uri (shell_view);
@@ -1929,37 +2055,16 @@ socket_destroy_cb (GtkWidget *socket_widget, gpointer data)
 	}
 
 	if (viewing_closed_uri)
-		e_shell_view_display_uri (shell_view, NULL);
+		e_shell_view_display_uri (shell_view, NULL, TRUE);
 
 	e_shell_component_maybe_crashed (priv->shell, uri, folder_type, shell_view);
 
 	/* We were actively viewing the component that just crashed, so flip to the default URI */
 	if (viewing_closed_uri)
-		e_shell_view_display_uri (shell_view, E_SHELL_VIEW_DEFAULT_URI);
+		e_shell_view_display_uri (shell_view, E_SHELL_VIEW_DEFAULT_URI, TRUE);
 }
 
 
-static const char *
-get_type_for_storage (EShellView *shell_view,
-		      const char *name,
-		      const char **physical_uri_return)
-{
-	EShellViewPrivate *priv;
-	EStorageSet *storage_set;
-	EStorage *storage;
-
-	priv = shell_view->priv;
-
-	storage_set = e_shell_get_storage_set (priv->shell);
-	storage = e_storage_set_get_storage (storage_set, name);
-	if (!storage)
-		return NULL;
-
-	*physical_uri_return = e_storage_get_toplevel_node_uri (storage);
-
-	return e_storage_get_toplevel_node_type (storage);
-}
-
 static const char *
 get_type_for_folder (EShellView *shell_view,
 		     const char *path,
@@ -1985,7 +2090,8 @@ get_type_for_folder (EShellView *shell_view,
 /* Create a new view for @uri with @control.  It assumes a view for @uri does not exist yet.  */
 static View *
 get_view_for_uri (EShellView *shell_view,
-		  const char *uri)
+		  const char *uri,
+		  const char *view_info)
 {
 	EShellViewPrivate *priv;
 	CORBA_Environment ev;
@@ -1997,7 +2103,6 @@ get_view_for_uri (EShellView *shell_view,
 	GtkWidget *socket;
 	Bonobo_Control corba_control;
 	const char *path;
-	const char *slash;
 	const char *physical_uri;
 	const char *folder_type;
 	int destroy_connection_id;
@@ -2012,13 +2117,8 @@ get_view_for_uri (EShellView *shell_view,
 	if (*path == '\0')
 		return NULL;
 
-	/* FIXME: This code needs to be made more robust.  */
-	slash = strchr (path + 1, G_DIR_SEPARATOR);
-	if (slash == NULL || slash[1] == '\0')
-		folder_type = get_type_for_storage (shell_view, path + 1, &physical_uri);
-	else
-		folder_type = get_type_for_folder (shell_view, path, &physical_uri);
-	if (folder_type == NULL)
+	folder_type = get_type_for_folder (shell_view, path, &physical_uri);
+	if (folder_type == NULL || physical_uri == NULL)
 		return NULL;
 
 	folder_type_registry = e_shell_get_folder_type_registry (e_shell_view_get_shell (shell_view));
@@ -2031,7 +2131,7 @@ get_view_for_uri (EShellView *shell_view,
 
 	CORBA_exception_init (&ev);
 
-	corba_control = GNOME_Evolution_ShellComponent_createView (handler, physical_uri, folder_type, &ev);
+	corba_control = GNOME_Evolution_ShellComponent_createView (handler, physical_uri, folder_type, view_info, &ev);
 
 	if (ev._major != CORBA_NO_EXCEPTION) {
 		CORBA_exception_free (&ev);
@@ -2085,7 +2185,8 @@ show_existing_view (EShellView *shell_view,
 
 static gboolean
 create_new_view_for_uri (EShellView *shell_view,
-			 const char *uri)
+			 const char *uri,
+			 const char *view_info)
 {
 	View *view;
 	EShellViewPrivate *priv;
@@ -2093,7 +2194,7 @@ create_new_view_for_uri (EShellView *shell_view,
 
 	priv = shell_view->priv;
 
-	view = get_view_for_uri (shell_view, uri);
+	view = get_view_for_uri (shell_view, uri, view_info);
 	if (view == NULL)
 		return FALSE;
 
@@ -2113,19 +2214,46 @@ create_new_view_for_uri (EShellView *shell_view,
 	return TRUE;
 }
 
-gboolean
-e_shell_view_display_uri (EShellView *shell_view,
-			  const char *uri)
+static char *
+evolution_uri_for_default_uri (EShell *shell,
+			       const char *default_uri)
+{
+	char *uri;
+	char *path;
+	char *extra;
+
+	if (! e_shell_parse_uri (shell, default_uri, &path, &extra))
+		return NULL;
+
+	uri = g_strconcat (E_SHELL_URI_PREFIX, path, "#", extra, NULL);
+
+	g_free (path);
+	g_free (extra);
+
+	return uri;
+}
+
+static gboolean
+display_uri (EShellView *shell_view,
+	     const char *uri,
+	     gboolean add_to_history,
+	     gboolean queue)
 {
 	EShellViewPrivate *priv;
 	View *view;
 	gboolean retval;
-
-	g_return_val_if_fail (shell_view != NULL, FALSE);
-	g_return_val_if_fail (E_IS_SHELL_VIEW (shell_view), FALSE);
+	const char *view_info;
+	char *real_uri;
+	char *allocated_uri = NULL;
 
 	priv = shell_view->priv;
 
+	if (uri == NULL && priv->uri == NULL)
+		return TRUE;
+
+	if (priv->uri != NULL && uri != NULL && strcmp (priv->uri, uri) == 0)
+		return TRUE;
+
 	bonobo_window_freeze (BONOBO_WINDOW (shell_view));
 
 	if (uri == NULL) {
@@ -2135,33 +2263,59 @@ e_shell_view_display_uri (EShellView *shell_view,
 		set_current_notebook_page (shell_view, 0);
 
 		g_free (priv->uri);
-		priv->uri = NULL;
+		priv->uri = real_uri = NULL;
 
 		retval = TRUE;
 		goto end;
 	}
 
+	if (strncmp (uri, E_SHELL_DEFAULTURI_PREFIX, E_SHELL_DEFAULTURI_PREFIX_LEN) == 0) {
+		allocated_uri = evolution_uri_for_default_uri (e_shell_view_get_shell (shell_view), uri);
+		uri = allocated_uri;
+	}
+
+	view_info = strchr (uri, '#');
+	if (view_info) {
+		real_uri = g_strndup (uri, view_info - uri);
+		view_info++;
+	} else {
+		view_info = "";
+		real_uri = g_strdup (uri);
+	}
+
 	if (strncmp (uri, E_SHELL_URI_PREFIX, E_SHELL_URI_PREFIX_LEN) != 0) {
 		retval = FALSE;
 		goto end;
 	}
 
-	view = g_hash_table_lookup (priv->uri_to_view, uri);
+	view = g_hash_table_lookup (priv->uri_to_view, real_uri);
 	if (view != NULL) {
-		show_existing_view (shell_view, uri, view);
-	} else if (! create_new_view_for_uri (shell_view, uri)) {
+		show_existing_view (shell_view, real_uri, view);
+	} else if (! create_new_view_for_uri (shell_view, real_uri, view_info)) {
+		if (! queue) {
+			retval = FALSE;
+			goto end;
+		}
+
 		cleanup_delayed_selection (shell_view);
-		priv->delayed_selection = g_strdup (uri);
+		priv->delayed_selection = g_strdup (real_uri);
 		gtk_signal_connect_full (GTK_OBJECT (e_shell_get_storage_set (priv->shell)),
 					 "new_folder", GTK_SIGNAL_FUNC (new_folder_cb), NULL,
 					 shell_view, NULL, FALSE, TRUE);
-		retval = FALSE;
+		retval = TRUE;
 		goto end;
 	}
 
 	retval = TRUE;
 
  end:
+	g_free (real_uri);
+
+	if (add_to_history && retval == TRUE && priv->uri != NULL)
+		e_history_add (priv->history, g_strdup (priv->uri));
+
+	update_navigation_buttons (shell_view);
+
 	g_free (priv->set_folder_uri);
 	priv->set_folder_uri = NULL;
 
@@ -2174,9 +2328,28 @@ e_shell_view_display_uri (EShellView *shell_view,
 
 	bonobo_window_thaw (BONOBO_WINDOW (shell_view));
 
+	gtk_signal_emit (GTK_OBJECT (shell_view), signals[VIEW_CHANGED],
+			 e_shell_view_get_current_path (shell_view),
+			 e_shell_view_get_current_uri (shell_view),
+			 e_shell_view_get_current_folder_type (shell_view),
+			 e_shell_view_get_current_component_id (shell_view));
+
+	g_free (allocated_uri);
+
 	return retval;
 }
 
+gboolean
+e_shell_view_display_uri (EShellView *shell_view,
+			  const char *uri,
+			  gboolean queue)
+{
+	g_return_val_if_fail (shell_view != NULL, FALSE);
+	g_return_val_if_fail (E_IS_SHELL_VIEW (shell_view), FALSE);
+
+	return display_uri (shell_view, uri, TRUE, queue);
+}
+
 
 void
 e_shell_view_show_shortcut_bar (EShellView *shell_view,
@@ -2234,8 +2407,8 @@ e_shell_view_show_folder_bar (EShellView *shell_view,
 		e_title_bar_set_button_mode (E_TITLE_BAR (priv->storage_set_title_bar),
 					     E_TITLE_BAR_BUTTON_MODE_CLOSE);
 
-		e_shell_folder_title_bar_set_clickable (E_SHELL_FOLDER_TITLE_BAR (priv->folder_title_bar),
-							FALSE);
+		e_shell_folder_title_bar_set_title_clickable (E_SHELL_FOLDER_TITLE_BAR (priv->folder_title_bar),
+							      FALSE);
 	} else {
 		if (GTK_WIDGET_VISIBLE (priv->storage_set_view_box)) {
 			/* FIXME this is a private field!  */
@@ -2248,8 +2421,8 @@ e_shell_view_show_folder_bar (EShellView *shell_view,
 		e_title_bar_set_button_mode (E_TITLE_BAR (priv->storage_set_title_bar),
 					     E_TITLE_BAR_BUTTON_MODE_PIN);
 
-		e_shell_folder_title_bar_set_clickable (E_SHELL_FOLDER_TITLE_BAR (priv->folder_title_bar),
-							TRUE);
+		e_shell_folder_title_bar_set_title_clickable (E_SHELL_FOLDER_TITLE_BAR (priv->folder_title_bar),
+							      TRUE);
 	}
 
         priv->folder_bar_shown = !! show;
@@ -2258,6 +2431,21 @@ e_shell_view_show_folder_bar (EShellView *shell_view,
 			 priv->folder_bar_shown);
 }
 
+void
+e_shell_view_show_settings (EShellView *shell_view)
+{
+	EShellViewPrivate *priv;	
+	const char *type;
+
+	g_return_if_fail (shell_view != NULL);
+	g_return_if_fail (E_IS_SHELL_VIEW (shell_view));
+	
+	priv = shell_view->priv;
+	
+	type = e_shell_view_get_current_folder_type (shell_view);
+	e_shell_show_settings (priv->shell, type, shell_view);
+}
+
 gboolean
 e_shell_view_shortcut_bar_shown (EShellView *shell_view)
 {
@@ -2399,6 +2587,29 @@ e_shell_view_get_current_folder_type (EShellView *shell_view)
 	return get_type_for_folder (shell_view, current_path, NULL);
 }
 
+const char *
+e_shell_view_get_current_component_id (EShellView *shell_view)
+{
+	EShellViewPrivate *priv;
+	EFolderTypeRegistry *type_registry;
+	EvolutionShellComponentClient *component_client;
+	const char *current_folder_type;
+
+	priv = shell_view->priv;
+
+	type_registry = e_shell_get_folder_type_registry (priv->shell);
+
+	current_folder_type = e_shell_view_get_current_folder_type (shell_view);
+	if (current_folder_type == NULL)
+		return NULL;
+
+	component_client = e_folder_type_registry_get_handler_for_type (type_registry, current_folder_type);
+	if (component_client == NULL)
+		return NULL;
+
+	return evolution_shell_component_client_get_id (component_client);
+}
+
 
 /**
  * e_shell_view_save_settings:
@@ -2525,9 +2736,10 @@ e_shell_view_load_settings (EShellView *shell_view,
 	Bonobo_ConfigDatabase db;
 	EShellViewPrivate *priv;
 	EShortcutBar *shortcut_bar;
-	int num_groups, group, val;
+	int num_groups, val;
 	long width, height;
 	char *stringval, *prefix, *filename, *key;
+	CORBA_Environment ev;
 
 	g_return_val_if_fail (shell_view != NULL, FALSE);
 	g_return_val_if_fail (E_IS_SHELL_VIEW (shell_view), FALSE);
@@ -2541,9 +2753,14 @@ e_shell_view_load_settings (EShellView *shell_view,
 
 	prefix = g_strdup_printf ("/Shell/Views/%d/", view_num);
 
+	CORBA_exception_init (&ev);
 	key = g_strconcat (prefix, "Width", NULL);
-	width = bonobo_config_get_long (db, key, NULL);
+	width = bonobo_config_get_long (db, key, &ev);
 	g_free (key);
+	if (ev._major != CORBA_NO_EXCEPTION) {
+		CORBA_exception_free (&ev);
+		return FALSE;
+	}
 
 	key = g_strconcat (prefix, "Height", NULL);
 	height = bonobo_config_get_long (db, key, NULL);
@@ -2580,30 +2797,24 @@ e_shell_view_load_settings (EShellView *shell_view,
 	priv->view_hpaned_position = val;
 	g_free (key);
 
-	key = g_strconcat (prefix, "DisplayedURI", NULL);
-	stringval = bonobo_config_get_string (db, key, NULL);
-	if (stringval) {
-		if (! e_shell_view_display_uri (shell_view, stringval))
-			e_shell_view_display_uri (shell_view, E_SHELL_VIEW_DEFAULT_URI);
-	} else
-		e_shell_view_display_uri (shell_view, E_SHELL_VIEW_DEFAULT_URI);
+	if (priv->uri == NULL && priv->delayed_selection == NULL) {
+		key = g_strconcat (prefix, "DisplayedURI", NULL);
+		stringval = bonobo_config_get_string (db, key, NULL);
+		if (stringval) {
+			if (! e_shell_view_display_uri (shell_view, stringval, FALSE)) {
+				e_shell_view_display_uri (shell_view, E_SHELL_VIEW_DEFAULT_URI, FALSE);
+				e_shell_view_display_uri (shell_view, stringval, TRUE);
+			}
+		} else {
+			e_shell_view_display_uri (shell_view, E_SHELL_VIEW_DEFAULT_URI, TRUE);
+		}
 
-	g_free (stringval);
-	g_free (key);
+		g_free (stringval);
+		g_free (key);
+	}
 
 	num_groups = e_shortcut_model_get_num_groups (shortcut_bar->model);
 
-	for (group = 0; group < num_groups; group++) {
-		int iconmode;
-
-		key = g_strdup_printf ("%sShortcutBarGroup%dIconMode", prefix,
-				       group);
-		iconmode = bonobo_config_get_long (db, key, NULL);
-		g_free (key);
-
-		e_shortcut_bar_set_view_type (shortcut_bar, group, iconmode);
-	}
-
 	g_free (prefix);
 
 	/* Load the expanded state for the ShellView's StorageSetView */
