@@ -289,8 +289,6 @@ folder_selection_dialog_folder_selected_cb (EShellFolderSelectionDialog *folder_
 	g_free (corba_folder.evolutionUri);
 
 	CORBA_exception_free (&ev);
-
-	gtk_widget_destroy (GTK_WIDGET (folder_selection_dialog));
 }
 
 
@@ -651,6 +649,16 @@ impl_Shell_setLineStatus (PortableServer_Servant servant,
 		e_shell_go_offline (shell, NULL);
 }
 
+
+void
+e_shell_set_interactive (EShell *shell,
+			 gboolean interactive)
+{
+	g_return_if_fail (E_IS_SHELL (shell));
+
+	set_interactive (shell, interactive);
+}
+
 
 /* Set up the ::Activity interface.  */
 
@@ -905,6 +913,32 @@ set_owner_on_components (EShell *shell,
 	e_free_string_list (id_list);
 }
 
+
+/* EStorageSet callbacks.  */
+
+static void
+storage_set_moved_folder_callback (EStorageSet *storage_set,
+				   const char *source_path,
+				   const char *destination_path,
+				   void *data)
+{
+	EShell *shell;
+	char *source_uri;
+	char *destination_uri;
+
+	shell = E_SHELL (data);
+
+	source_uri = g_strconcat (E_SHELL_URI_PREFIX, source_path, NULL);
+	destination_uri = g_strconcat (E_SHELL_URI_PREFIX, destination_path, NULL);
+
+	e_shortcuts_update_shortcuts_for_changed_uri (e_shell_get_shortcuts (shell),
+						      source_uri,
+						      destination_uri);
+
+	g_free (source_uri);
+	g_free (destination_uri);
+}
+
 
 /* EShellView handling and bookkeeping.  */
 
@@ -916,24 +950,9 @@ view_delete_event_cb (GtkWidget *widget,
 	EShell *shell;
 
 	g_assert (E_IS_SHELL_VIEW (widget));
-
 	shell = E_SHELL (data);
-	e_shell_save_settings (shell);
 
-	if (g_list_length (shell->priv->views) != 1) {
-		/* If it's not the last view, just destroy it.  */
-		return FALSE;
-	}
-
-	if (shell->priv->preparing_to_quit)
-		return TRUE;
-
-	/* If it's the last view, ask for confirm before actually destroying
-	   this view.  */
-	if (e_shell_prepare_for_quit (shell))
-		return FALSE;
-	else
-		return TRUE;
+	return ! e_shell_request_close_view (shell, E_SHELL_VIEW (widget));
 }
 
 static void
@@ -1215,6 +1234,11 @@ e_shell_construct (EShell *shell,
 	priv->folder_type_registry = e_folder_type_registry_new ();
 	priv->uri_schema_registry  = e_uri_schema_registry_new ();
 	priv->storage_set          = e_storage_set_new (priv->folder_type_registry);
+
+	gtk_signal_connect_while_alive (GTK_OBJECT (priv->storage_set), "moved_folder",
+					GTK_SIGNAL_FUNC (storage_set_moved_folder_callback),
+					shell,
+					GTK_OBJECT (shell));
 	
 	e_folder_type_registry_register_type (priv->folder_type_registry,
 					      "noselect", "empty.gif",
@@ -1444,6 +1468,30 @@ e_shell_create_view_from_uri_and_settings (EShell *shell,
 	return view;
 }
 
+gboolean
+e_shell_request_close_view (EShell *shell,
+			    EShellView *shell_view)
+{
+	g_return_val_if_fail (E_IS_SHELL (shell), FALSE);
+	g_return_val_if_fail (E_IS_SHELL_VIEW (shell_view), FALSE);
+
+	e_shell_save_settings (shell);
+
+	if (g_list_length (shell->priv->views) != 1) {
+		/* Not the last view.  */
+		return TRUE;
+	}
+
+	if (shell->priv->preparing_to_quit)
+		return FALSE;
+
+	/* If it's the last view, ask for confirm. */
+	if (e_shell_prepare_for_quit (shell))
+		return TRUE;
+	else
+		return FALSE;
+}
+
 
 /**
  * e_shell_get_local_directory:
