@@ -132,6 +132,13 @@ enum {
 
 static unsigned int signals[LAST_SIGNAL] = { 0 };
 
+
+/* Forward declarations.  */
+
+static void setup_folder_changed_callbacks (EStorageSetView *storage_set_view,
+					    EFolder *folder,
+					    const char *path);
+
 
 /* DND stuff.  */
 
@@ -776,6 +783,7 @@ popup_folder_menu (EStorageSetView *storage_set_view,
 	priv = storage_set_view->priv;
 
 	folder = e_storage_set_get_folder (priv->storage_set, priv->right_click_row_path);
+	gtk_object_ref (GTK_OBJECT (folder));
 
 	folder_type_registry = e_storage_set_get_folder_type_registry (priv->storage_set);
 	g_assert (folder_type_registry != NULL);
@@ -811,6 +819,7 @@ popup_folder_menu (EStorageSetView *storage_set_view,
 										 e_folder_get_physical_uri (folder),
 										 e_folder_get_type_string (folder));
 
+	gtk_object_unref (GTK_OBJECT (folder));
 	gtk_widget_destroy (GTK_WIDGET (menu));
 
 	e_tree_right_click_up (E_TREE (storage_set_view));
@@ -1422,7 +1431,7 @@ etree_value_at (ETreeModel *etree,
 	storage_set = priv->storage_set;
 
 	/* Storages are always highlighted. */
-	if (path_is_storage (etree, tree_path) && col == 1)
+	if (col == 1 && path_is_storage (etree, tree_path))
 		return (void *) TRUE;
 
 	path = (char *) e_tree_memory_node_get_data (E_TREE_MEMORY(etree), tree_path);
@@ -1713,6 +1722,10 @@ new_folder_cb (EStorageSet *storage_set,
 		e_tree_memory_node_remove (E_TREE_MEMORY(etree), new_node);
 		return;
 	}
+
+	setup_folder_changed_callbacks (storage_set_view,
+					e_storage_set_get_folder (storage_set, path),
+					path);
 }
 
 static void
@@ -1777,7 +1790,7 @@ class_init (EStorageSetViewClass *klass)
 	GtkObjectClass *object_class;
 	ETreeClass *etree_class;
 
-	parent_class = gtk_type_class (e_tree_get_type ());
+	parent_class = gtk_type_class (PARENT_TYPE);
 
 	object_class = GTK_OBJECT_CLASS (klass);
 	object_class->destroy = impl_destroy;
@@ -1936,6 +1949,62 @@ folder_changed_cb (EFolder *folder,
 	e_tree_model_node_data_changed (priv->etree_model, node);
 }
 
+static void
+folder_name_changed_cb (EFolder *folder,
+			void *data)
+{
+	EStorageSetView *storage_set_view;
+	EStorageSetViewPrivate *priv;
+	FolderChangedCallbackData *callback_data;
+	ETreePath parent_node;
+	const char *last_separator;
+	char *parent_path;
+
+	callback_data = (FolderChangedCallbackData *) data;
+
+	storage_set_view = callback_data->storage_set_view;
+	priv = storage_set_view->priv;
+
+	last_separator = strrchr (callback_data->path, E_PATH_SEPARATOR);
+
+	parent_path = g_strndup (callback_data->path, last_separator - callback_data->path);
+	parent_node = g_hash_table_lookup (priv->path_to_etree_node, parent_path);
+	g_free (parent_path);
+
+	if (parent_node == NULL) {
+		g_warning ("EStorageSetView -- EFolder::name_changed emitted for a folder whose path I don't know.");
+		return;
+	}
+
+	e_tree_memory_sort_node (E_TREE_MEMORY (priv->etree_model), parent_node,
+				 folder_sort_callback, storage_set_view);
+}
+
+static void
+setup_folder_changed_callbacks (EStorageSetView *storage_set_view,
+				EFolder *folder,
+				const char *path)
+{
+	FolderChangedCallbackData *folder_changed_callback_data;
+
+	folder_changed_callback_data = g_new (FolderChangedCallbackData, 1);
+	folder_changed_callback_data->storage_set_view = storage_set_view;
+	folder_changed_callback_data->path = g_strdup (path);
+
+	gtk_signal_connect_while_alive (GTK_OBJECT (folder), "name_changed",
+					GTK_SIGNAL_FUNC (folder_name_changed_cb),
+					folder_changed_callback_data,
+					GTK_OBJECT (storage_set_view));
+
+	e_gtk_signal_connect_full_while_alive (GTK_OBJECT (folder), "changed",
+					       GTK_SIGNAL_FUNC (folder_changed_cb),
+					       NULL,
+					       folder_changed_callback_data,
+					       folder_changed_callback_data_destroy_notify,
+					       FALSE, FALSE,
+					       GTK_OBJECT (storage_set_view));
+}
+
 
 static void
 insert_folders (EStorageSetView *storage_set_view,
@@ -1960,7 +2029,6 @@ insert_folders (EStorageSetView *storage_set_view,
 		return;
 
 	for (p = folder_path_list; p != NULL; p = p->next) {
-		FolderChangedCallbackData *folder_changed_callback_data;
 		EFolder *folder;
 		const char *folder_name;
 		const char *folder_path;
@@ -1971,23 +2039,14 @@ insert_folders (EStorageSetView *storage_set_view,
 		folder_name = e_folder_get_name (folder);
 
 		full_path = g_strconcat ("/", storage_name, folder_path, NULL);
+
+		setup_folder_changed_callbacks (storage_set_view, folder, full_path);
+
 		node = e_tree_memory_node_insert (E_TREE_MEMORY(etree), parent, -1, (void *) full_path);
 		e_tree_memory_sort_node(E_TREE_MEMORY(etree), parent, folder_sort_callback, storage_set_view);
 		add_node_to_hash (storage_set_view, full_path, node);
 
 		insert_folders (storage_set_view, node, storage, folder_path);
-
-		folder_changed_callback_data = g_new (FolderChangedCallbackData, 1);
-		folder_changed_callback_data->storage_set_view = storage_set_view;
-		folder_changed_callback_data->path = g_strdup (full_path);
-
-		e_gtk_signal_connect_full_while_alive (GTK_OBJECT (folder), "changed",
-						       GTK_SIGNAL_FUNC (folder_changed_cb),
-						       NULL,
-						       folder_changed_callback_data,
-						       folder_changed_callback_data_destroy_notify,
-						       FALSE, FALSE,
-						       GTK_OBJECT (storage_set_view));
 	}
 
 	e_free_string_list (folder_path_list);
@@ -2293,9 +2352,6 @@ void
 e_storage_set_view_enable_search (EStorageSetView *storage_set_view,
 				  gboolean enable)
 {
-	ETableState *state;
-	ETableSortColumn col;
-
 	g_return_if_fail (storage_set_view != NULL);
 	g_return_if_fail (E_IS_STORAGE_SET_VIEW (storage_set_view));
 
@@ -2305,17 +2361,7 @@ e_storage_set_view_enable_search (EStorageSetView *storage_set_view,
 		return;
 	
 	storage_set_view->priv->search_enabled = enable;
-
-	state = e_tree_get_state_object (E_TREE (storage_set_view));
-
-	if (enable) {
-		col.column = 0;
-		col.ascending = TRUE;
-		e_table_sort_info_sorting_set_nth (state->sort_info, 0, col);
-	} else
-		e_table_sort_info_sorting_truncate (state->sort_info, 0);
-
-	e_tree_set_state_object (E_TREE (storage_set_view), state);
+	e_tree_set_search_column (E_TREE (storage_set_view), enable ? 0 : -1);
 }
 
 void
