@@ -4,9 +4,8 @@
  * Copyright (C) 2000  Ximian, Inc.
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * modify it under the terms of version 2 of the GNU General Public
+ * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -46,6 +45,7 @@ enum {
 	NEW_FOLDER,
 	UPDATE_FOLDER,
 	REMOVED_FOLDER,
+	HAS_SUBFOLDERS,
 	LAST_SIGNAL
 };
 static guint signals[LAST_SIGNAL] = { 0 };
@@ -95,7 +95,6 @@ impl_GNOME_Evolution_StorageListener_notifyFolderCreated (PortableServer_Servant
 static void
 impl_GNOME_Evolution_StorageListener_notifyFolderUpdated (PortableServer_Servant servant,
 							  const CORBA_char *path,
-							  const CORBA_char *display_name,
 							  CORBA_long unread_count,
 							  CORBA_Environment *ev)
 {
@@ -106,7 +105,7 @@ impl_GNOME_Evolution_StorageListener_notifyFolderUpdated (PortableServer_Servant
 	priv = listener->priv;
 
 	gtk_signal_emit (GTK_OBJECT (listener), signals[UPDATE_FOLDER], path,
-			 display_name);
+			 unread_count);
 }
 
 static void
@@ -123,6 +122,21 @@ impl_GNOME_Evolution_StorageListener_notifyFolderRemoved (PortableServer_Servant
 	gtk_signal_emit (GTK_OBJECT (listener), signals[REMOVED_FOLDER], path);
 }
 
+static void
+impl_GNOME_Evolution_StorageListener_notifyHasSubfolders (PortableServer_Servant servant,
+							  const CORBA_char *path,
+							  const CORBA_char *message,
+							  CORBA_Environment *ev)
+{
+	EvolutionStorageListener *listener;
+	EvolutionStorageListenerPrivate *priv;
+
+	listener = gtk_object_from_servant (servant);
+	priv = listener->priv;
+
+	gtk_signal_emit (GTK_OBJECT (listener), signals[HAS_SUBFOLDERS], path, message);
+}
+
 static EvolutionStorageListenerServant *
 create_servant (EvolutionStorageListener *listener)
 {
@@ -226,6 +240,7 @@ corba_class_init (void)
 	epv->notifyFolderCreated = impl_GNOME_Evolution_StorageListener_notifyFolderCreated;
 	epv->notifyFolderUpdated = impl_GNOME_Evolution_StorageListener_notifyFolderUpdated;
 	epv->notifyFolderRemoved = impl_GNOME_Evolution_StorageListener_notifyFolderRemoved;
+	epv->notifyHasSubfolders = impl_GNOME_Evolution_StorageListener_notifyHasSubfolders;
 
 	vepv = & my_GNOME_Evolution_StorageListener_vepv;
 	vepv->_base_epv                     = base_epv;
@@ -262,10 +277,10 @@ class_init (EvolutionStorageListenerClass *klass)
 						  GTK_RUN_FIRST,
 						  object_class->type,
 						  GTK_SIGNAL_OFFSET (EvolutionStorageListenerClass, update_folder),
-						  gtk_marshal_NONE__POINTER_POINTER,
+						  gtk_marshal_NONE__POINTER_INT,
 						  GTK_TYPE_NONE, 2,
 						  GTK_TYPE_STRING,
-						  GTK_TYPE_STRING);
+						  GTK_TYPE_INT);
 
 	signals[REMOVED_FOLDER] = gtk_signal_new ("removed_folder",
 						  GTK_RUN_FIRST,
@@ -275,6 +290,15 @@ class_init (EvolutionStorageListenerClass *klass)
 						  GTK_TYPE_NONE, 1,
 						  GTK_TYPE_STRING);
 
+	signals[HAS_SUBFOLDERS] = gtk_signal_new ("has_subfolders",
+						  GTK_RUN_FIRST,
+						  object_class->type,
+						  GTK_SIGNAL_OFFSET (EvolutionStorageListenerClass, has_subfolders),
+						  gtk_marshal_NONE__POINTER_POINTER,
+						  GTK_TYPE_NONE, 2,
+						  GTK_TYPE_STRING,
+						  GTK_TYPE_STRING);
+
 	gtk_object_class_add_signals (object_class, signals, LAST_SIGNAL);
 
 	corba_class_init ();
